#include "Unit.h"
#include "tinyxml.h"

bool MonsterCompare(const Units::Monster& m1, const Units::Monster& m2)
{
	return (m1.level < m2.level) || (m1.level == m2.level && m1.exp < m2.exp);
}


int ParseMonsterXML(string path, Units *units)
{
	TiXmlDocument tilesetXml(path.c_str());
	if (!(tilesetXml.LoadFile())) 
		return LEVGEN_FILEOPENERROR;

	TiXmlNode* root = tilesetXml.FirstChild(LEVGEN_UNIT_DEFINITION);
	if (!root) 
		return LEVGEN_INVALID_UNIT_XML;

	TiXmlElement* heroxml = root->FirstChildElement(LEVGEN_UNIT_HERO);
	if (!heroxml) 
		return LEVGEN_INVALID_UNIT_XML;

	TiXmlElement* heroimagexml = heroxml->FirstChildElement(LEVGEN_UNIT_IMAGE);

	for (int i = 0; i < NDIRECTION; i++)
	{
		units->heroImagePath[i].clear();
	}

	while(heroimagexml)
	{
		for (int i = 0; i < NDIRECTION; i++)
		{
			if (strcmp(STRDIRECTION[i], heroimagexml->Attribute(LEVGEN_UNIT_ATTR_DIRECTION)) == 0)
			{
				units->heroImagePath[i] = heroimagexml->Attribute(LEVGEN_UNIT_ATTR_PATH);
			}
		}

		heroimagexml = heroimagexml->NextSiblingElement(LEVGEN_UNIT_IMAGE);
	}

	TiXmlElement* monsterxml = root->FirstChildElement(LEVGEN_UNIT_MONSTER);
	while(monsterxml)
	{
		Units::Monster newm;

		newm.id = monsterxml->Attribute(LEVGEN_UNIT_ATTR_ID);
		newm.level = atoi(monsterxml->Attribute(LEVGEN_UNIT_ATTR_LEVEL));
		newm.exp = atoi(monsterxml->Attribute(LEVGEN_UNIT_ATTR_EXP));
		newm.imagePath = monsterxml->Attribute(LEVGEN_UNIT_ATTR_IMAGEPATH);
		units->monsters.push_back(newm);

		monsterxml = monsterxml->NextSiblingElement(LEVGEN_UNIT_MONSTER);
	}

	return LEVGEN_NOERROR;

}

