#pragma once

#include <tchar.h>
#include <string>
#include <vector>
#include <deque>
#include <algorithm>
#include <assert.h>
#include <iostream>
#include"levgen_error.h"

using namespace std;

// max level size
const int MAX_TILE_SIZE				= 15;
const int MAX_SLOT_MATRIX_SIZE		= 100;

// for tuning
const int		STRAIGHT_PRIORITY			= 70;
const double	RELEVANCE_TOLERANCE_RATIO	= 1;
const int		MARGIN_CONNECTION_PERCENT	= 50;
const int		EXPAND_MAINPATH_PERCENT		= 80;
const int		MINIMUM_SECTION_LENGTH		= 4;
const int		ARTIPOINT_MARGIN_AT_END		= 3;
const int		MONSTER_EXISTENCE_PERCENT	= 5;
const int		RETRY_COUNT					= 10;
const int		MAX_LENGTH_FOR_SECTION_DIFF[]	= {0, 5, 7, 5, 7, 7, 9, 7, 9, 9, 9};
const int		NUM_SUBPATH_FOR_SECTION_DIFF[]	= {0, 2, 3, 3, 3, 4, 4, 4, 4, 5};

// for exp
const int		EXP_FOR_GET_KEY				= 0;
const int		EXP_FOR_UNLOCK_PER_COMPLEX	= 500;
const int		EXP_FOR_UNLOCK_PER_KEY		= 300;
const int		EXP_FOR_SOLVE_SOKOBAN		= 300;


struct Direction
{
	enum Type
	{
		North = 0,
		West = 1,
		East = 2,
		South = 3,
		End = 4
	};
};

struct Path
{
	enum Type
	{
		MainPath	= 0,
		ExtendedMP	= 1,
		SubPath		= 2,
		Undefined	= 3
	};
};

struct ArtiPoint
{
	enum Type
	{
		Lock = 0,
		Sokoban = 1,
		End = 2
	};
	Type	type;
	int		sequence;
	int		index;
};

struct Coor
{
	int x;
	int y;
};

struct CoorwDir
{
	int x;
	int y;
	Direction::Type dir;
};


struct Slot
{
	int		sequence;
	bool	isSokoban;
	bool	isUsed;
	int		tileindex;
	int		section;
	bool	adjacencies[Direction::End];
	int		orgSequence;
	Path::Type pathType;
};

const int DELTAX[] = {0, -1, 1, 0};
const int DELTAY[] = {-1, 0, 0, 1};
const int NDIRECTION = 4;
const Direction::Type DIRECTIONS[] = {Direction::North, Direction::West, Direction::East, Direction::South};
const char* const STRDIRECTION[4] = { "North", "West", "East", "South" };


