#include "..\common\LevelGenResult.h"
#include "CelledMapGenerator.h"


int CCelledMapGenerator::Process(CKeyLockComposer *pKeyLockComposer)
{
	m_coorSet.clear();
	m_pKeyLockComposer = pKeyLockComposer;
	m_pTileArranger = m_pKeyLockComposer->m_pTileArranger;
	m_pTileContainer = m_pTileArranger->m_pTileContainer;

	//  m_pKeyLockComposer   key  Ҵ.
	for (int i = 0; i < (int) (m_pKeyLockComposer->m_keys.size()); i++)
	{
		// cell  key  ϵ ǥ ߿ ҴѴ.
		//    ִ.
		//   Ʒ  Ÿ ִٰ .
		// -2  1  1  1  -2
		// -2 -2 -2 -2  -2
		// -2 -2 -2 -2  -2
		// -2 -2  1  1  -2
		// -2 -2  1  1  -2
		//  solution path  ʿ  Ǵµ
		// Ʒʿ key  ġصθ δ key     ȴ.
		// ̸ ϱ  slot  solution path  ϸ鼭 ÿ key ġѴ.
		Key newkey;
		newkey.x = -1;
		newkey.y = -1;
		newkey.relLockIndex = m_pKeyLockComposer->m_keys[i].relLockIndex;
		m_cellKeys.push_back(newkey);
	}

	//  m_pKeyLockComposer   Lock CellLock ȯ.
	for (int i = 0; i < (int) (m_pKeyLockComposer->m_locks.size()); i++)
	{
		Lock& lock = m_pKeyLockComposer->m_locks[i];

		CellLock newlock;
		newlock.dir = lock.dir;
		newlock.numOfKeys = lock.numOfKeys;

		int x1 = 0, y1 = 0;
		int x2 = 0, y2 = 0;

		if (newlock.dir == Direction::North)
		{
			x1 = 0; 
			x2 = m_pTileContainer->m_tileWidth - 1;
			y1 = 0;
			y2 = 0;
		}
		else if (newlock.dir == Direction::West)
		{
			x1 = 0; 
			x2 = 0;
			y1 = 0;
			y2 = m_pTileContainer->m_tileHeight - 1;
		}
		else if (newlock.dir == Direction::East)
		{
			x1 = m_pTileContainer->m_tileWidth - 1;
			x2 = m_pTileContainer->m_tileWidth - 1;
			y1 = 0;
			y2 = m_pTileContainer->m_tileHeight - 1;
		}
		else if (newlock.dir == Direction::South)
		{
			x1 = 0; 
			x2 = m_pTileContainer->m_tileWidth - 1;
			y1 = m_pTileContainer->m_tileHeight - 1;
			y2 = m_pTileContainer->m_tileHeight - 1;
		}

		bool flag = false;
		for (int k = y1; k <= y2; k++)
		{
			for (int l = x1; l <= x2; l++)
			{
				int tileindex = m_pTileArranger->m_slotMatrix[lock.y][lock.x].tileindex;
				int celltype = m_pTileContainer->m_tiles[tileindex].typeMatrix[k][l];

				if (celltype > 0)
				{
					if (!flag)
					{
						flag = true;
						GetCelledCoor(lock.x, lock.y, l, k, &(newlock.x1), &(newlock.y1));
					}
					GetCelledCoor(lock.x, lock.y, l, k, &(newlock.x2), &(newlock.y2));
				}
			}
		}

		for (int j = 0; j < (int) (lock.relKeyIndexes.size()); j++)
		{
			newlock.relKeyIndexes.push_back(lock.relKeyIndexes[j]);
		}

		m_cellLocks.push_back(newlock);
	}

	int rtv = AllocateSokobanBlock();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = BuildSlotSolutionPath();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = BuildCellSolutionPath();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = AllocateMonsters();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	return LEVGEN_NOERROR;
}

void CCelledMapGenerator::GetCelledCoor(int slotX, int slotY, int inSlotX, int inSlotY, int *x, int *y)
{
	*x = slotX * (m_pTileContainer->m_tileWidth) + inSlotX;
	*y = slotY * (m_pTileContainer->m_tileHeight) + inSlotY;
}


void CCelledMapGenerator::Export(ostream& os)
{
	os << "<" << LEVGEN_RESULT_LEVEL_DEFINITION << ">" << endl;
	{
		os << "\t<" << LEVGEN_RESULT_TILEALLOCATION;
		os << " " << LEVGEN_RESULT_ATTR_WIDTH << "=\"" << m_pTileArranger->m_userCondition.width << "\"";
		os << " " << LEVGEN_RESULT_ATTR_HEIGHT << "=\"" << m_pTileArranger->m_userCondition.height << "\"";
		os << " " << LEVGEN_RESULT_ATTR_TILESETPATH << "=\"" << m_pTileArranger->m_userCondition.tilesetXMLPath << "\"";
		os << " >" << endl;
		for (int i = 0; i < m_pTileArranger->m_userCondition.height; i++)
		{
			os << "\t\t<![" << LEVGEN_RESULT_CDATA << "[";
			for (int j = 0; j < m_pTileArranger->m_userCondition.width; j++)
			{
				int tileindex = m_pTileArranger->m_slotMatrix[i][j].tileindex;

				os << " " << m_pTileContainer->m_tiles[tileindex].id.c_str();
			}
			os << " ]]>" << endl;
		}
		os << "\t</" << LEVGEN_RESULT_TILEALLOCATION << ">" << endl;
	}	
	{
		os << "\t<" << LEVGEN_RESULT_LOCKKEYALLOCATION << ">" << endl;
		{
			for (int i = 0; i < (int) (m_cellLocks.size()); i++)
			{
				os << "\t\t<" << LEVGEN_RESULT_LOCK;
				os << " " << LEVGEN_RESULT_ATTR_DIRECTION << "=\"" << STRDIRECTION[m_cellLocks[i].dir] << "\"";
				{
					os << " " << LEVGEN_RESULT_ATTR_X1 << "=\"" << m_cellLocks[i].x1 << "\"";
					os << " " << LEVGEN_RESULT_ATTR_X2 << "=\"" << m_cellLocks[i].x2 << "\"";
				}

				{
					os << " " << LEVGEN_RESULT_ATTR_Y1 << "=\"" << m_cellLocks[i].y1 << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y2 << "=\"" << m_cellLocks[i].y2 << "\"";
				}
				os << " >" << endl;
				for (int j = 0; j < (int) (m_cellLocks[i].relKeyIndexes.size()); j++)
				{
					int keyindex = m_cellLocks[i].relKeyIndexes[j];
					os << "\t\t\t<" << LEVGEN_RESULT_KEY;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_cellKeys[keyindex].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_cellKeys[keyindex].y << "\"";
					os << " />" << endl;
				}
				os << "\t\t</" << LEVGEN_RESULT_LOCK << ">" << endl;
			}
		}
		os << "\t</" << LEVGEN_RESULT_LOCKKEYALLOCATION << ">" << endl;
	}

	{
		os << "\t<" << LEVGEN_RESULT_MONSTERALLOCATION << ">" << endl;
		{
			for (int j = 0; j < (int) (m_monsterAllocations.size()); j++)
			{
				os << "\t\t<" << LEVGEN_RESULT_MONSTER;
				os << " " << LEVGEN_RESULT_ATTR_ID << "=\"" << m_monsterAllocations[j].id << "\"";
				os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAllocations[j].x << "\"";
				os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAllocations[j].y << "\"";
				os << " />" << endl;
			}
		}
		os << "\t</" << LEVGEN_RESULT_MONSTERALLOCATION << ">" << endl;
	}

	{
		os << "\t<" << LEVGEN_RESULT_SOKOBAN << ">" << endl;
		{
			for (int j = 0; j < (int) (m_sokobanBlocks.size()); j++)
			{
				os << "\t\t<" << LEVGEN_RESULT_BLOCK;
				os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_sokobanBlocks[j].x << "\"";
				os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_sokobanBlocks[j].y << "\"";
				os << " />" << endl;
			}
		}
		os << "\t</" << LEVGEN_RESULT_SOKOBAN << ">" << endl;
	}

	cout << "<" << "Slot path" << ">" << endl;
	for (int i = 0; i < (int) (m_slotSolutionPath.size()); i++)
	{
		cout << m_slotSolutionPath[i].x << " ";
		cout << m_slotSolutionPath[i].y << " ";
		cout << m_slotSolutionPath[i].type << " ";
		cout << m_slotSolutionPath[i].exp << " ";
		cout << endl;
	}

	{
		os << "\t<" << LEVGEN_RESULT_SOLUTIONPATH;
		os << " " << LEVGEN_RESULT_ATTR_STARTX << "=\"" << m_monsterAddedCellSolutionPath[0].x << "\"";
		os << " " << LEVGEN_RESULT_ATTR_STARTY << "=\"" << m_monsterAddedCellSolutionPath[0].y << "\"";
		os << " " << LEVGEN_RESULT_ATTR_GOALX << "=\"" << m_monsterAddedCellSolutionPath[m_monsterAddedCellSolutionPath.size()-1].x << "\"";
		os << " " << LEVGEN_RESULT_ATTR_GOALY << "=\"" << m_monsterAddedCellSolutionPath[m_monsterAddedCellSolutionPath.size()-1].y << "\"";
		os << " >" << endl;


		{
			for (int i = 0; i < (int) (m_monsterAddedCellSolutionPath.size()); i++)
			{
				if (m_monsterAddedCellSolutionPath[i].type == Action::Move)
				{
					os << "\t\t<" << LEVGEN_RESULT_ACTION;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAddedCellSolutionPath[i].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAddedCellSolutionPath[i].y << "\"";
					os << " " << LEVGEN_RESULT_ATTR_TYPE << "=\"" << LEVGEN_RESULT_ATTR_V_MOVE << "\"";
					os << " " << LEVGEN_RESULT_ATTR_EXP << "=\"" << m_monsterAddedCellSolutionPath[i].exp << "\"";
					os << " />" << endl;
				}
				else if (m_monsterAddedCellSolutionPath[i].type == Action::SolveSokoban)
				{
					os << "\t\t<" << LEVGEN_RESULT_ACTION;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAddedCellSolutionPath[i].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAddedCellSolutionPath[i].y << "\"";
					os << " " << LEVGEN_RESULT_ATTR_TYPE << "=\"" << LEVGEN_RESULT_ATTR_V_SOLVESOKOBAN << "\"";
					os << " " << LEVGEN_RESULT_ATTR_EXP << "=\"" << m_monsterAddedCellSolutionPath[i].exp << "\"";
					os << " >" << endl;
					for (int j = 0; j < (int) (m_sokobanBlocks.size()); j++)
					{
						if (m_sokobanBlocks[j].relArtiPointIndex == m_monsterAddedCellSolutionPath[i].relIndex)
						{
							os << "\t\t\t<" << LEVGEN_RESULT_BLOCK;
							os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_sokobanBlocks[j].x << "\"";
							os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_sokobanBlocks[j].y << "\"";
							os << " />" << endl;
						}
					}
					os << "\t\t</" << LEVGEN_RESULT_ACTION << ">" << endl;
				}
				else if (m_monsterAddedCellSolutionPath[i].type == Action::GetKey)
				{
					os << "\t\t<" << LEVGEN_RESULT_ACTION;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAddedCellSolutionPath[i].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAddedCellSolutionPath[i].y << "\"";
					os << " " << LEVGEN_RESULT_ATTR_TYPE << "=\"" << LEVGEN_RESULT_ATTR_V_GETKEY << "\"";
					os << " " << LEVGEN_RESULT_ATTR_EXP << "=\"" << m_monsterAddedCellSolutionPath[i].exp << "\"";
					os << " />" << endl;
				}
				else if (m_monsterAddedCellSolutionPath[i].type == Action::UnLock)
				{
					os << "\t\t<" << LEVGEN_RESULT_ACTION;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAddedCellSolutionPath[i].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAddedCellSolutionPath[i].y << "\"";
					os << " " << LEVGEN_RESULT_ATTR_TYPE << "=\"" << LEVGEN_RESULT_ATTR_V_UNLOCK << "\"";
					os << " " << LEVGEN_RESULT_ATTR_EXP << "=\"" << m_monsterAddedCellSolutionPath[i].exp << "\"";
					os << " >" << endl;
					CellLock& celllock = m_cellLocks[m_monsterAddedCellSolutionPath[i].relIndex];
					for (int k = celllock.x1; k <= celllock.x2; k++)
					{
						for (int l = celllock.y1; l <= celllock.y2; l++)
						{
							os << "\t\t\t<" << LEVGEN_RESULT_LOCK;
							os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << k << "\"";
							os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << l << "\"";
							os << " " << LEVGEN_RESULT_ATTR_DIRECTION << "=\"" << STRDIRECTION[celllock.dir] << "\"";
							os << " />" << endl;
						}

					}
					os << "\t\t</" << LEVGEN_RESULT_ACTION << ">" << endl;
				}
				else if (m_monsterAddedCellSolutionPath[i].type == Action::KillMonster)
				{
					os << "\t\t<" << LEVGEN_RESULT_ACTION;
					os << " " << LEVGEN_RESULT_ATTR_X << "=\"" << m_monsterAddedCellSolutionPath[i].x << "\"";
					os << " " << LEVGEN_RESULT_ATTR_Y << "=\"" << m_monsterAddedCellSolutionPath[i].y << "\"";
					os << " " << LEVGEN_RESULT_ATTR_TYPE << "=\"" << LEVGEN_RESULT_ATTR_V_KILLMONSTER << "\"";
					os << " " << LEVGEN_RESULT_ATTR_EXP << "=\"" << m_monsterAddedCellSolutionPath[i].exp << "\"";
					os << " />" << endl;
				}
			}
		}
		os << "\t</" << LEVGEN_RESULT_SOLUTIONPATH << ">" << endl;
	}
	os << "</" << LEVGEN_RESULT_LEVEL_DEFINITION << ">" << endl;
}

bool CCelledMapGenerator::IsAttainbleCell(int slotX, int slotY, int inSlotX, int inSlotY)
{
	int tileindex = m_pTileArranger->m_slotMatrix[slotY][slotX].tileindex;
	int celltype = m_pTileContainer->m_tiles[tileindex].typeMatrix[inSlotY][inSlotX];

	if (celltype > 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}

int CCelledMapGenerator::BuildSlotSolutionPath()
{
	m_slotSolutionPath.clear();

	Action startAction;
	startAction.x = m_pTileArranger->m_startX;
	startAction.y = m_pTileArranger->m_startY;
	startAction.exp = 0;
	startAction.type = Action::Move;

	m_slotSolutionPath.push_back(startAction);

	TiledGraph g;

	m_pKeyLockComposer->ConstructGraph(&g);

	// ־ ׷   ø   ´.
	// 켱 Lock  ´.
	for (int i = 0; i < (int) (m_pKeyLockComposer->m_locks.size()); i++)
	{
		Lock& lock = m_pKeyLockComposer->m_locks[i];

		g.adjacency[lock.y][lock.x][lock.dir] = EdgeInTiledGraph::Locked;
		g.adjacency[lock.y+DELTAY[lock.dir]][lock.x+DELTAX[lock.dir]][(Direction::Type) (3-lock.dir)] = EdgeInTiledGraph::Locked;
	}

	//  ߿ Lock    Ƿ Sokoban  ´.
	for (int i = 0; i < (int) (m_pTileArranger->m_artiPoints.size()); ++i)
	{
		if (m_pTileArranger->m_artiPoints[i].type == ArtiPoint::Sokoban)
		{
			// arti point ̿Ͽ  ´.
			int x,y;
			int apSequence = m_pTileArranger->m_artiPoints[i].sequence;
			bool rbool = m_pTileArranger->FindCoorWithSequenceOnMainPath(apSequence, &x, &y);
			if (!rbool)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			for (int k = 0; k < NDIRECTION; k++) 
			{
				Direction::Type dir = DIRECTIONS[k];
				if (m_pTileArranger->m_slotMatrix[y][x].adjacencies[dir])
				{
					int nextX = x+DELTAX[dir];
					int nextY = y+DELTAY[dir];
					if ( m_pTileArranger->m_slotMatrix[nextY][nextX].pathType == Path::MainPath 
						&& m_pTileArranger->m_slotMatrix[nextY][nextX].sequence == (apSequence - 1))
					{
						if (g.adjacency[y][x][dir] == EdgeInTiledGraph::Disconnected)
							return LEVGEN_INVALID_GRAPH_CONNECTION;

						if (g.adjacency[nextY][nextX][(Direction::Type) (3 - dir)]  == EdgeInTiledGraph::Disconnected)
							return LEVGEN_INVALID_GRAPH_CONNECTION;

						g.adjacency[y][x][dir] =  EdgeInTiledGraph::Locked;
						g.adjacency[nextY][nextX][(Direction::Type) (3 - dir)] = EdgeInTiledGraph::Locked;
					}
				}
			}
		}
	}

	int lastLock = 0;
	
	// section  solution path ã´.
	for (int i = 0; i < (int) (m_pTileArranger->m_artiPoints.size()); ++i)
	{
		if (m_pTileArranger->m_artiPoints[i].type == ArtiPoint::Lock)
		{
			int relLockIndex = -1;
			int artiX;
			int artiY;

			int apSequence = m_pTileArranger->m_artiPoints[i].sequence;
			bool rbool = m_pTileArranger->FindCoorWithSequenceOnMainPath(apSequence, &artiX, &artiY);
			if (!rbool)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			for (int j = 0; j < (int) (m_pKeyLockComposer->m_locks.size()); j++)
			{
				if (m_pKeyLockComposer->m_locks[j].x == artiX && m_pKeyLockComposer->m_locks[j].y == artiY)
				{
					relLockIndex = j;
					break;
				}
			}

			if (relLockIndex==-1)
				return LEVGEN_NO_LOCK_RELATED_AP;

			//  lastLock relLockIndex  Ǯ ȴ.
			for (int j = lastLock; j <= relLockIndex; j++)
			{
				Lock& lock = m_pKeyLockComposer->m_locks[j];
				// 켱 key  ã´.
				for (int k = 0; k < (int) (m_pKeyLockComposer->m_locks[j].numOfKeys); k++)
				{
					int keyIndex = m_pKeyLockComposer->m_locks[j].relKeyIndexes[k];
					int cursor = m_slotSolutionPath.size() - 1;

					AppendSlotSolutionPath(g, m_slotSolutionPath[cursor].x, m_slotSolutionPath[cursor].y,
						m_pKeyLockComposer->m_keys[keyIndex].x, m_pKeyLockComposer->m_keys[keyIndex].y);

					{
						// ⼭ key cellǥ ҴѴ.
						int lastindex = m_slotSolutionPath.size() - 1;
						Direction::Type dir = Direction::End;

						dir = (Direction::Type) (3- (GetDirectionFromTwoCoor(m_slotSolutionPath[lastindex - 1].x, m_slotSolutionPath[lastindex - 1].y,
														m_slotSolutionPath[lastindex].x, m_slotSolutionPath[lastindex].y)));

						int rtv = AllocateKey(keyIndex, dir);
						if (rtv != LEVGEN_NOERROR)
							return LEVGEN_KEY_ALLOC_FAILED;
					}

					Action getKeyAction;
					getKeyAction.x = m_pKeyLockComposer->m_keys[keyIndex].x;
					getKeyAction.y = m_pKeyLockComposer->m_keys[keyIndex].y;
					getKeyAction.relIndex = keyIndex;
					getKeyAction.type = Action::GetKey;
					getKeyAction.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp + EXP_FOR_GET_KEY;
					m_slotSolutionPath.push_back(getKeyAction);
				}

				int nextx = lock.x + DELTAX[lock.dir];
				int nexty = lock.y + DELTAY[lock.dir];
				//   lock Ǭ.
				// lock   lock.x  lock.y  lock  ǵ Ƿ nextx, nexty ؾ 
				int cursor = m_slotSolutionPath.size() - 1;
				AppendSlotSolutionPath(g, m_slotSolutionPath[cursor].x, m_slotSolutionPath[cursor].y,
					nextx, nexty);

				Action unlockAction;
				unlockAction.x = nextx;
				unlockAction.y = nexty;
				unlockAction.relIndex = j;
				unlockAction.type = Action::UnLock;
				unlockAction.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp 
									+ EXP_FOR_UNLOCK_PER_COMPLEX * lock.complexity
									+ EXP_FOR_UNLOCK_PER_KEY * lock.numOfKeys;
				m_slotSolutionPath.push_back(unlockAction);

				// Ǭ lock graph  ش.
				//    lock Ѵ.
				//  section  Ѿ ڿ ׷ Ž 
				//  section  Ѿ ʿ䰡  ̴.
				if (j != relLockIndex)
				{
					Direction::Type nextdir = (Direction::Type) (3 - lock.dir);
					if (g.adjacency[lock.y][lock.x][lock.dir] != EdgeInTiledGraph::Locked)
						return LEVGEN_INVALID_GRAPH_CONNECTION;
					if (g.adjacency[nexty][nextx][nextdir] != EdgeInTiledGraph::Locked)
						return LEVGEN_INVALID_GRAPH_CONNECTION;
					g.adjacency[lock.y][lock.x][lock.dir] = EdgeInTiledGraph::Connected;
					g.adjacency[nexty][nextx][nextdir] = EdgeInTiledGraph::Connected;
				}
			}
			lastLock = relLockIndex + 1;

			//  articulation point  Ѿ.
			Action moveAction;
			moveAction.x = artiX;
			moveAction.y = artiY;
			moveAction.relIndex = -1;
			moveAction.type = Action::Move;
			moveAction.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp; 
			m_slotSolutionPath.push_back(moveAction);
		}
		else if (m_pTileArranger->m_artiPoints[i].type == ArtiPoint::Sokoban)
		{
			int artiX;
			int artiY;

			int apSequence = m_pTileArranger->m_artiPoints[i].sequence;
			bool rbool = m_pTileArranger->FindCoorWithSequenceOnMainPath(apSequence, &artiX, &artiY);
			if (!rbool)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			bool flag = false;
			int beforeArtiX = 0;
			int beforeArtiY = 0;
			for (int k = 0; k < NDIRECTION; k++) 
			{
				Direction::Type dir = DIRECTIONS[k];
				if (m_pTileArranger->m_slotMatrix[artiY][artiX].adjacencies[dir])
				{
					beforeArtiX = artiX+DELTAX[dir];
					beforeArtiY = artiY+DELTAY[dir];
					if ( m_pTileArranger->m_slotMatrix[beforeArtiY][beforeArtiX].pathType == Path::MainPath 
						&& m_pTileArranger->m_slotMatrix[beforeArtiY][beforeArtiX].sequence == (apSequence - 1))
					{
						flag = true;
						break;
					}
				}
			}

			if (!flag)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			// sokoban ٷ  slot ̵Ͽ sokoban Ǭ.
			int cursor = m_slotSolutionPath.size() - 1;
			AppendSlotSolutionPath(g, m_slotSolutionPath[cursor].x, m_slotSolutionPath[cursor].y,
				beforeArtiX, beforeArtiY);


			Action solveSokobanAction;
			solveSokobanAction.x = artiX;
			solveSokobanAction.y = artiY;
			solveSokobanAction.relIndex = i;
			solveSokobanAction.type = Action::SolveSokoban;
			solveSokobanAction.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp + EXP_FOR_SOLVE_SOKOBAN;
			m_slotSolutionPath.push_back(solveSokobanAction);

			//  articulation point  Ѿ.
			Action moveAction;
			moveAction.x = artiX;
			moveAction.y = artiY;
			moveAction.relIndex = -1;
			moveAction.type = Action::Move;
			moveAction.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp; 
			m_slotSolutionPath.push_back(moveAction);
		}
	}

	//  goal .
	int cursor = m_slotSolutionPath.size() - 1;
	AppendSlotSolutionPath(g, m_slotSolutionPath[cursor].x, m_slotSolutionPath[cursor].y,
		m_pTileArranger->m_endX, m_pTileArranger->m_endY);

	return LEVGEN_NOERROR;
}


int	CCelledMapGenerator::AllocateKey(int index, Direction::Type dir)
{
	int slotX = m_pKeyLockComposer->m_keys[index].x;
	int slotY = m_pKeyLockComposer->m_keys[index].y;

	int gateInSlotX = -1;
	int gateInSlotY = -1;
	m_pTileContainer->GetGate(dir, &gateInSlotX, &gateInSlotY);

	int tileindex = m_pTileArranger->m_slotMatrix[slotY][slotX].tileindex;

	Coor startCoor;
	startCoor.x = gateInSlotX;
	startCoor.y = gateInSlotY;

	bool visited[MAX_TILE_SIZE][MAX_TILE_SIZE];
	memset((void*) visited, 0, sizeof(visited));

	deque<Coor> queOfCoor;
	vector<Coor> candidates;
	queOfCoor.clear();
	candidates.clear();

	queOfCoor.push_back(startCoor);
	while(!queOfCoor.empty())
	{
		Coor curCoor = queOfCoor.front();
		queOfCoor.pop_front();

		if (m_coorSet.find(curCoor) == m_coorSet.end())
		{
			candidates.push_back(curCoor);
		}

		for (int k = 0; k < NDIRECTION; k++) 
		{
			Direction::Type dir = DIRECTIONS[k];

			Coor newCoor;
			newCoor.x = curCoor.x + DELTAX[dir];
			newCoor.y = curCoor.y + DELTAY[dir];

			if (newCoor.x < 0 || newCoor.x >= m_pTileContainer->m_tileWidth
				|| newCoor.y < 0 || newCoor.y >= m_pTileContainer->m_tileHeight)
				continue;

			if (m_pTileContainer->m_tiles[tileindex].typeMatrix[newCoor.y][newCoor.x] <= 0) continue;
			if (visited[newCoor.y][newCoor.x]) continue;

			visited[newCoor.y][newCoor.x] = true;
			queOfCoor.push_back(newCoor);
		}
	}

	if (candidates.empty())
		return LEVGEN_KEY_ALLOC_FAILED;

	int randsel = rand() % candidates.size();

	GetCelledCoor(slotX, slotY, candidates[randsel].x, candidates[randsel].y, &(m_cellKeys[index].x), &(m_cellKeys[index].y));

	Coor newcoor;
	newcoor.x = m_cellKeys[index].x;
	newcoor.y = m_cellKeys[index].y;

	m_coorSet.insert(newcoor);

	return LEVGEN_NOERROR;
}

int	CCelledMapGenerator::AppendSlotSolutionPath(TiledGraph &g, int startX, int startY, int goalX, int goalY)
{
	if (startX == goalX && startY == goalY)
		return LEVGEN_NOERROR;

	Coor startCoor;
	startCoor.x = startX;
	startCoor.y = startY;

	Direction::Type from[MAX_SLOT_MATRIX_SIZE][MAX_SLOT_MATRIX_SIZE];

	for (int i = 0; i < m_pTileArranger->m_userCondition.height; i++)
	{
		for (int j = 0; j < m_pTileArranger->m_userCondition.width; j++)
		{
			from[i][j] = Direction::End;
		}
	}

	deque<Coor> queOfCoor;
	bool		finished = false;
	queOfCoor.clear();

	queOfCoor.push_back(startCoor);
	while(!queOfCoor.empty())
	{
		Coor curCoor = queOfCoor.front();
		queOfCoor.pop_front();

		for (int k = 0; k < NDIRECTION; k++) 
		{
			Direction::Type dir = DIRECTIONS[k];

			if (g.adjacency[curCoor.y][curCoor.x][dir] != EdgeInTiledGraph::Connected) continue;

			Coor newCoor;
			newCoor.x = curCoor.x + DELTAX[dir];
			newCoor.y = curCoor.y + DELTAY[dir];

			if (from[newCoor.y][newCoor.x] != Direction::End) continue;


			from[newCoor.y][newCoor.x] = (Direction::Type) (3 - dir);

			if (newCoor.x == goalX && newCoor.y == goalY)
			{
				finished = true;
				break;
			}

			queOfCoor.push_back(newCoor);
		}

		if (finished)
			break;
	}

	if (!finished)
		return LEVGEN_NO_TILED_SOLUTION_PATH;

	vector<Coor> backtraces;
	backtraces.clear();

	Coor current;
	current.x = goalX;
	current.y = goalY;
	while(current.y != startY || current.x != startX)
	{
		if (from[current.y][current.x] == Direction::End)
			return LEVGEN_INVALID_OP_IN_FIND_TILED_SOLUTION_PATH;

		backtraces.push_back(current);

		Coor next;
		next.x = current.x + DELTAX[from[current.y][current.x]];
		next.y = current.y + DELTAY[from[current.y][current.x]];

		current = next;
	}

	for (int i = backtraces.size() - 1; i >=0; i--)
	{
		Action move;
		move.x = backtraces[i].x;
		move.y = backtraces[i].y;
		move.type = Action::Move;
		move.relIndex = -1;
		move.exp = m_slotSolutionPath[m_slotSolutionPath.size() - 1].exp; 
		m_slotSolutionPath.push_back(move);
	}

	return LEVGEN_NOERROR;
}

int CCelledMapGenerator::AllocateSokobanBlock()
{
	for (int i = 0; i < (int) (m_pTileArranger->m_artiPoints.size()); ++i)
	{
		if (m_pTileArranger->m_artiPoints[i].type == ArtiPoint::Sokoban)
		{
			int artiX;
			int artiY;

			int apSequence = m_pTileArranger->m_artiPoints[i].sequence;
			bool rbool = m_pTileArranger->FindCoorWithSequenceOnMainPath(apSequence, &artiX, &artiY);
			if (!rbool)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			if (m_pTileArranger->m_slotMatrix[artiY][artiX].isSokoban == false)
				return LEVGEN_INVALID_SOKOBAN_ALLOCATION;

			int tileindex = m_pTileArranger->m_slotMatrix[artiY][artiX].tileindex;
			for (int k = 0; k < (int) (m_pTileContainer->m_tiles[tileindex].sokobanBlocks.size()); k++)
			{
				Coor& inTilecoor = m_pTileContainer->m_tiles[tileindex].sokobanBlocks[k];

				SokobanBlock cur;
				
				cur.x = artiX * (m_pTileContainer->m_tileWidth) + inTilecoor.x;
				cur.y = artiY * (m_pTileContainer->m_tileHeight) + inTilecoor.y;
				cur.relArtiPointIndex = i;

				m_sokobanBlocks.push_back(cur);
			}

		}
	}

	return LEVGEN_NOERROR;
}

int CCelledMapGenerator::BuildCellSolutionPath()
{
	int startX = -1;
	int startY = -1;

	int prevSlotX = -1;
	int prevSlotY = -1;

	int curInSlotX = -1;
	int curInSlotY = -1;

	for (int i = 0; i < (int) (m_slotSolutionPath.size()); i++)
	{
		Action& slotAction = m_slotSolutionPath[i];

		if (slotAction.type == Action::Move)
		{
			if (prevSlotX != -1)
			{
				//  slot  ̹ slot  
				Direction::Type dir = Direction::End;

				dir = GetDirectionFromTwoCoor(prevSlotX, prevSlotY, slotAction.x, slotAction.y);

				int exitInPrevSlotX = -1;
				int exitInPrevSlotY = -1;
				m_pTileContainer->GetGate(dir, &exitInPrevSlotX, &exitInPrevSlotY);

				if (curInSlotX != -1)
				{
					int rtv = AppendCellSolutionPath(curInSlotX, curInSlotY,
												exitInPrevSlotX, exitInPrevSlotY, prevSlotX, prevSlotY);
					if (rtv != LEVGEN_NOERROR)
						return rtv;
				}
				else
				{
					// Ⱑ start
					GetCelledCoor(prevSlotX, prevSlotY, exitInPrevSlotX, exitInPrevSlotY, &startX, &startY);

					Action newaction;
					newaction.x = startX;
					newaction.y = startY;
					newaction.exp = 0;
					newaction.type = Action::Move;
					m_cellSolutionPath.push_back(newaction);
				}

				m_pTileContainer->GetGate((Direction::Type) (3 - dir), &curInSlotX, &curInSlotY);
				Action newaction;
				GetCelledCoor(slotAction.x, slotAction.y, curInSlotX, curInSlotY, &(newaction.x), &(newaction.y));
				newaction.exp = m_cellSolutionPath[m_cellSolutionPath.size()-1].exp;
				newaction.type = Action::Move;
				m_cellSolutionPath.push_back(newaction);
			}

			prevSlotX = slotAction.x;
			prevSlotY = slotAction.y;
		}
		else if (slotAction.type == Action::SolveSokoban)
		{
			Direction::Type dir = Direction::End;

			dir = GetDirectionFromTwoCoor(prevSlotX, prevSlotY, slotAction.x, slotAction.y);

			int exitInPrevSlotX = -1;
			int exitInPrevSlotY = -1;
			m_pTileContainer->GetGate(dir, &exitInPrevSlotX, &exitInPrevSlotY);

			int rtv = AppendCellSolutionPath(curInSlotX, curInSlotY,
										exitInPrevSlotX, exitInPrevSlotY, prevSlotX, prevSlotY);
			if (rtv != LEVGEN_NOERROR)
				return rtv;

			curInSlotX = exitInPrevSlotX;
			curInSlotY = exitInPrevSlotY;

			Action& lastaction = m_cellSolutionPath[m_cellSolutionPath.size()-1];
			Action newaction;
			newaction.x = lastaction.x;
			newaction.y = lastaction.y;
			newaction.type = Action::SolveSokoban;
			newaction.exp = lastaction.exp + (slotAction.exp - m_slotSolutionPath[i-1].exp);
			newaction.relIndex = slotAction.relIndex;
			m_cellSolutionPath.push_back(newaction);
		}
		else if (slotAction.type == Action::GetKey)
		{
			int keyInPrevSlotX;
			int keyInPrevSlotY;

			ConvertToInSlotCellCoor(m_cellKeys[slotAction.relIndex].x, m_cellKeys[slotAction.relIndex].y,
										&(keyInPrevSlotX), &(keyInPrevSlotY));

			int rtv = AppendCellSolutionPath(curInSlotX, curInSlotY,
										keyInPrevSlotX, keyInPrevSlotY, prevSlotX, prevSlotY);
			if (rtv != LEVGEN_NOERROR)
				return rtv;

			curInSlotX = keyInPrevSlotX;
			curInSlotY = keyInPrevSlotY;

			Action& lastaction = m_cellSolutionPath[m_cellSolutionPath.size()-1];
			Action newaction;
			newaction.x = lastaction.x;
			newaction.y = lastaction.y;
			newaction.type = Action::GetKey;
			newaction.exp = lastaction.exp + (slotAction.exp - m_slotSolutionPath[i-1].exp);
			newaction.relIndex = -1;
			m_cellSolutionPath.push_back(newaction);
		}
		else if (slotAction.type == Action::UnLock)
		{
			Direction::Type dir = (Direction::Type) (3 - m_cellLocks[slotAction.relIndex].dir);

			int exitInPrevSlotX = -1;
			int exitInPrevSlotY = -1;
			m_pTileContainer->GetGate(dir, &exitInPrevSlotX, &exitInPrevSlotY);

			int rtv = AppendCellSolutionPath(curInSlotX, curInSlotY,
										exitInPrevSlotX, exitInPrevSlotY, prevSlotX, prevSlotY);
			if (rtv != LEVGEN_NOERROR)
				return rtv;

			curInSlotX = exitInPrevSlotX;
			curInSlotY = exitInPrevSlotY;

			Action& lastaction = m_cellSolutionPath[m_cellSolutionPath.size()-1];
			Action newaction;
			newaction.x = lastaction.x;
			newaction.y = lastaction.y;
			newaction.type = Action::UnLock;
			newaction.exp = lastaction.exp + (slotAction.exp - m_slotSolutionPath[i-1].exp);
			newaction.relIndex = slotAction.relIndex;
			m_cellSolutionPath.push_back(newaction);
		}
	}

	return LEVGEN_NOERROR;
}

int CCelledMapGenerator::AppendCellSolutionPath(int startX, int startY, int goalX, int goalY, int slotX, int slotY)
{
	if (startX == goalX && startY == goalY)
		return LEVGEN_NOERROR;

	int tileindex = m_pTileArranger->m_slotMatrix[slotY][slotX].tileindex;
	string id = m_pTileContainer->m_tiles[tileindex].id;

	Coor startCoor;
	startCoor.x = startX;
	startCoor.y = startY;

	Direction::Type from[MAX_TILE_SIZE][MAX_TILE_SIZE];
	for (int i = 0; i < m_pTileContainer->m_tileHeight; i++)
	{
		for (int j = 0; j < m_pTileContainer->m_tileWidth; j++)
		{
			from[i][j] = Direction::End;
		}
	}

	deque<Coor> queOfCoor;
	bool		finished = false;
	queOfCoor.clear();

	queOfCoor.push_back(startCoor);
	while(!queOfCoor.empty())
	{
		Coor curCoor = queOfCoor.front();
		queOfCoor.pop_front();

		for (int k = 0; k < NDIRECTION; k++) 
		{
			Direction::Type dir = DIRECTIONS[k];

			Coor newCoor;
			newCoor.x = curCoor.x + DELTAX[dir];
			newCoor.y = curCoor.y + DELTAY[dir];

			if (newCoor.x < 0 || newCoor.x >= m_pTileContainer->m_tileWidth
				|| newCoor.y < 0 || newCoor.y >= m_pTileContainer->m_tileHeight)
				continue;

			if (m_pTileContainer->m_tiles[tileindex].typeMatrix[newCoor.y][newCoor.x] <= 0) continue;
			if (from[newCoor.y][newCoor.x] != Direction::End) continue;

			from[newCoor.y][newCoor.x] = (Direction::Type) (3 - dir);

			if (newCoor.x == goalX && newCoor.y == goalY)
			{
				finished = true;
				break;
			}
			queOfCoor.push_back(newCoor);
		}

		if (finished)
			break;
	}

	if (!finished)
		return LEVGEN_NO_CELL_SOLUTION_PATH_IN_A_TILE;

	vector<Coor> backtraces;
	backtraces.clear();

	Coor current;
	current.x = goalX;
	current.y = goalY;
	while(current.y != startY || current.x != startX)
	{
		if (from[current.y][current.x] == Direction::End)
			return LEVGEN_INVALID_OP_IN_FIND_CELL_SOLUTION_PATH;

		backtraces.push_back(current);

		Coor next;
		next.x = current.x + DELTAX[from[current.y][current.x]];
		next.y = current.y + DELTAY[from[current.y][current.x]];

		current = next;
	}

	for (int i = backtraces.size() - 1; i >=0; i--)
	{
		Action move;
		GetCelledCoor(slotX, slotY, backtraces[i].x, backtraces[i].y, &(move.x), &(move.y));
		move.type = Action::Move;
		move.relIndex = -1;
		move.exp = m_cellSolutionPath[m_cellSolutionPath.size() - 1].exp; 
		m_cellSolutionPath.push_back(move);
	}

	return LEVGEN_NOERROR;
}

void CCelledMapGenerator::ConvertToInSlotCellCoor(int cellX, int cellY, int *x, int *y)
{
	*x = cellX % (m_pTileContainer->m_tileWidth);
	*y = cellY % (m_pTileContainer->m_tileHeight);
}

int CCelledMapGenerator::AllocateMonsters()
{
	int rtv = ParseMonsterXML(m_pTileArranger->m_userCondition.unitXMLPath.c_str(), &m_units);
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	sort(m_units.monsters.begin(), m_units.monsters.end(), MonsterCompare);

	m_monsterAddedCellSolutionPath.clear();

	for (int i = 0; i < (int) (m_cellSolutionPath.size()); i++)
	{
		Action& action = m_cellSolutionPath[i];

		Action monsterAddedAction = action;
		if (i == 0)
		{
			monsterAddedAction.exp = action.exp;
		}
		else
		{
			monsterAddedAction.exp = m_monsterAddedCellSolutionPath[m_monsterAddedCellSolutionPath.size() - 1].exp
										+ (action.exp - m_cellSolutionPath[i-1].exp);
		}

		m_monsterAddedCellSolutionPath.push_back(monsterAddedAction);

		Coor coor;
		coor.x = action.x;
		coor.y = action.y;

		if (m_moveSet.find(coor) != m_moveSet.end()) continue;
		m_moveSet.insert(coor);

		if (action.type == Action::Move && i > 1 && i < (int) (m_cellSolutionPath.size() - 1))
		{	
			if (rand() % 100 < MONSTER_EXISTENCE_PERCENT)
			{
				// monster  
				int monsterId = (int) (m_units.monsters.size() * (i / (float) (m_cellSolutionPath.size())));
				MonsterAlloc alloc;
				alloc.x = coor.x;
				alloc.y = coor.y;
				alloc.id = m_units.monsters[monsterId].id;

				m_monsterAllocations.push_back(alloc);

				Action monsterKillAction;
				monsterKillAction.x = monsterAddedAction.x;
				monsterKillAction.y = monsterAddedAction.y;
				monsterKillAction.exp = monsterAddedAction.exp + m_units.monsters[monsterId].exp;
				monsterKillAction.type = Action::KillMonster;
				monsterKillAction.relIndex = -1;
				m_monsterAddedCellSolutionPath.push_back(monsterKillAction);
			}
		}


	}

	return LEVGEN_NOERROR;
}


Direction::Type CCelledMapGenerator::GetDirectionFromTwoCoor(int x1, int y1, int x2, int y2)
{
	Direction::Type dir;

	dir = Direction::End;

	if (y1 + 1 == y2)
	{
		dir = Direction::South;
	}
	else if (x1 + 1 == x2)
	{
		dir = Direction::East;
	}
	else if (x1 - 1 == x2)
	{
		dir = Direction::West;
	}
	else if (y1 - 1 == y2)
	{
		dir = Direction::North;
	}

	return dir;
}