#pragma once

#include "..\common\common.h"
#include "KeyLockComposer.h"
#include "..\common\Unit.h"
#include <hash_set>

/** 
* \brief Cell  Lock Ÿ.
*/
struct CellLock
{
	int				x1;
	int				y1;
	int				x2;
	int				y2;
	Direction::Type dir;
	int				numOfKeys;
	vector<int>		relKeyIndexes;
};

struct SokobanBlock
{
	int				x;
	int				y;
	int				relArtiPointIndex;
};

struct MonsterAlloc
{
	int				x;
	int				y;
	string			id;
};

/** 
* \brief solution  ϳ Ÿ.
*/
struct Action
{
	int			x;
	int			y;
	/** 
	* \brief  ġ
	*/
	int			exp;
	/** 
	* \brief  Lock̳ Key  index
	*/
	int			relIndex;
	enum Type
	{
		None	= 0,
		Move,
		GetKey,
		UnLock,
		SolveSokoban,
		KillMonster
	};
	Type		type;
};

class coorhasher : public stdext::hash_compare <Coor>
{
public:
  size_t operator() (const Coor& c) const
  {
    return c.x * 100 + c.y;
  }

  bool operator() (const Coor& c1, const Coor& c2) const
  {
    return (c1.x < c2.x) || ((c1.x == c2.x) && (c1.y < c2.y));
  }
};

typedef stdext::hash_set<Coor, coorhasher> COOR_SET;


//////////////////////////////////////////////////////////////////////////////////////////////////
/// \brief CTileArranger CKeyLockComposer  ޾Ƽ Cell  level Ѵ.
///
///  ܰ迡 slot  matrix ̿Ͽ level ǥѴ.
///  ܰ迡  slot tile replaceϿ cell matrix Ѵ.
/// 
//////////////////////////////////////////////////////////////////////////////////////////////////
class CCelledMapGenerator
{
public:
	/** 
	* \brief CKeyLockComposer instance ޾Ƽ cell  level Ѵ.
	* 
	* \param	pKeyLockComposer	CKeyLockComposer instance
	* \return	 ڵ
	*/
	int				Process(CKeyLockComposer *pKeyLockComposer);

	/** 
	* \brief level   XML Ѵ.
	*/
	void			Export(ostream& os);

private:
	/** 
	* \brief slot ǥ slot  cell ǥ ޾Ƽ ü level cell ǥ ȯѴ.
	*/
	void			GetCelledCoor(int slotX, int slotY, int inSlotX, int inSlotY, int *x, int *y);

	/** 
	* \brief ü level cell ǥ ޾Ƽ slot ο cell ǥ ȯѴ.
	*/
	void			ConvertToInSlotCellCoor(int cellX, int cellY, int *x, int *y);

	/** 
	* \brief slot  Ư cell   type ȯѴ.
	*/
	bool			IsAttainbleCell(int slotX, int slotY, int inSlotX, int inSlotY);

	/** 
	* \brief slot  cell ǥ key ҴѴ.
	* 
	* direction ڷ ʿ  slot  ⿡  
	*   cell  ޶ ̴.
	* 
	* \param	index	keyindex
	* \param	dir	Slot  
	* \return	 ڵ
	*/
	int				AllocateKey(int index, Direction::Type dir);

	/** 
	* \brief slot  cell ǥ sokoban block ġѴ.
	*/
	int				AllocateSokobanBlock();

	/** 
	* \brief slot  solution path Ѵ.
	*/
	int				BuildSlotSolutionPath();

	/** 
	* \brief  Slot ǥ Slot ̵ϴ θ solution path  ߰Ѵ.
	*/
	int				AppendSlotSolutionPath(TiledGraph &g, int startX, int startY, int goalX, int goalY);

	/** 
	* \brief cell  solution path Ѵ.
	*/
	int				BuildCellSolutionPath();

	/** 
	* \brief Slot ο  cell ǥ cell ̵ϴ θ solution path  ߰Ѵ.
	*/
	int				AppendCellSolutionPath(int startX, int startY, int goalX, int goalY, int slotX, int slotY);

	/** 
	* \brief cell  solution path  monster ġѴ.
	*/
	int				AllocateMonsters();

	Direction::Type GetDirectionFromTwoCoor(int x1, int y1, int x2, int y2);

private:
	CKeyLockComposer*		m_pKeyLockComposer;
	CTileArranger*			m_pTileArranger;
	CTileContainer*			m_pTileContainer;

	/** 
	* \brief key, lock, sokoban block   cell ߺǾ ġǴ     cell  
	*/
	COOR_SET				m_coorSet;
	vector<Key>				m_cellKeys;
	vector<CellLock>		m_cellLocks;
	vector<SokobanBlock>	m_sokobanBlocks;
	vector<Action>			m_slotSolutionPath;
	vector<Action>			m_cellSolutionPath;
	vector<Action>			m_monsterAddedCellSolutionPath;

	/** 
	* \brief cell  solution path  monster ġ  ̹ solution path ѹ Դ cell̸ monster ġ ʵ ϱ 
	*/
	COOR_SET				m_moveSet;
	Units					m_units;
	vector<MonsterAlloc>	m_monsterAllocations;
};
