#pragma once

#include "..\common\tile.h"
#include "TileArranger.h"

struct EdgeInTiledGraph
{
	enum Type
	{
		Disconnected = 0,
		Connected = 1,
		Locked = 2
	};
};

/** 
* \brief  Slot vertex̰  Slot  edge ִ ׷ Ÿ.
*/
struct TiledGraph
{
	/** 
	* \brief  Slot  踦 ǥ
	*/
	char	adjacency[MAX_SLOT_MATRIX_SIZE][MAX_SLOT_MATRIX_SIZE][NDIRECTION];
	int		width;
	int		height;
};

/** 
* \brief Lock Ÿ.
*/
struct Lock
{
	/** 
	* \brief Lock xǥ
	*/
	int				x;
	/** 
	* \brief Lock yǥ
	*/
	int				y;
	/** 
	* \brief  ǥ Slot Lock ϴ .  Lock Slot Slot edge 
	*/
	Direction::Type dir;
	/** 
	* \brief  key 
	*/
	int				numOfKeys;
	/** 
	* \brief ø Ƚ.
	*/
	int				complexity;
	/** 
	* \brief  key vector<Key> index
	*/
	vector<int>		relKeyIndexes;
};

/** 
* \brief Key Ÿ.
*/
struct Key
{
	/** 
	* \brief Key xǥ
	*/
	int			x;
	/** 
	* \brief Key yǥ
	*/
	int			y;
	/** 
	* \brief Key  Lock vector<Lock>  index
	*/
	int			relLockIndex;
};

/** 
* \brief Bridge   ׷   ̻ иǴ Edge̴.
*/
typedef CoorwDir Bridge;

//////////////////////////////////////////////////////////////////////////////////////////////////
/// \brief CTileArranger  Graph  Lock Key ġѴ.
//////////////////////////////////////////////////////////////////////////////////////////////////
class CKeyLockComposer
{
	friend class CCelledMapGenerator;
public:
	/** 
	* \brief CKeyLockComposer member function ȣϿ  ǿ ´ key-lock ġѴ.
	* 
	* \param	pTileArranger	CTileArranger pointer
	* \return	 ڵ
	*/
	int				Process(CTileArranger *pTileArranger);
	void			PrintMatrix(ostream& os);

private:
	/** 
	* \brief m_pTileArrangerκ slotmatrix о tiledgraph  Ѵ.
	* 
	* \param	tiledGraph	 graph ȯ pointer
	*/
	void			ConstructGraph(TiledGraph *tiledGraph);

	/** 
	* \brief ο Lock ҴѴ.
	* 
	* ڷ ־ LockDef ´ Lock
	* (rootX, rootY) κ   ġ ġѴ.
	* 
	* \param	lockdef	 Lock 
	* \param	rootX	Graph  ΰ ϱ ϴ  xǥ
	* \param	rootY	Graph  ΰ ϱ ϴ  yǥ
	* \param	g		Lock ġؾ ϴ graph
	* \param	newLock	 ġ Lock Ǹ ȯ pointer
	* \return	 ڵ
	*/
	int				AllocateLock(LockDef lockdef, int rootX, int rootY, TiledGraph &g, Lock *newLock);

	/** 
	* \brief ο Key ҴѴ.
	* 
	* (rootX, rootY) κ   ġ Key ġѴ.
	* 
	* \param	rootX	Graph  ΰ ϱ ϴ  xǥ
	* \param	rootY	Graph  ΰ ϱ ϴ  yǥ
	* \param	g		Key ġؾ ϴ graph
	* \param	newKey	 ġ Key Ǹ ȯ pointer
	* \return	 ڵ
	*/
	int				AllocateKey(int rootX, int rootY, TiledGraph &g, Key *newKey);

	/**
	* \brief Է ־ graph    lock شϴ edge  tiledGraph ȯѴ.
	* 
	* \param	tileGraph	 Tiled Graph
	* \param	lock	graph  ϴ Lock
	* \param	newGraph	  Tiled Graph
	* \return	 ڵ
	*/
	int				GetCutGraph(TiledGraph &tileGraph, Lock &lock, TiledGraph *newGraph);

	/**
	* \brief ־ Graph Bridge ã´.
	*
	* \param	g	tiled graph	
	* \param	rootX	 켱 Ž  Slot xǥ
	* \param	rootY	 켱 Ž  Slot yǥ
	* \param	bridges	߰ Bridge  ȯ pointer
	* \return	 ڵ
	*/
	void			FindBridges(TiledGraph &g, int rootX, int rootY, vector<Bridge> *bridges);

	/**
	* \brief  켱 Ž  Articulation Point Bridge ã´.
	*
	* \param	g	tiled graph	
	* \param	x	 Ž  Slot xǥ
	* \param	y	 Ž  Slot yǥ
	* \param	parentX	θ Slot xǥ
	* \param	parentY	θ Slot yǥ
	* \param	isRoot	 Ž  Slot  켱 Ž Ʈ Ʈ 
	* \return	 ڵ
	*/
	int				ADFS(TiledGraph &g, int x, int y, int parentX, int parentY, bool isRoot);

	/**
	* \brief ο Lock ġ   óѴ.
	*
	*  ġ Lock ø Lock 쿡 ο Lock ġϰ  ʿ  Lock Key ġؾ Ѵ.
	* ø Lock ƴ϶ Lock ٱʿ Key ġϴ  ʿϴ.
	*
	* \param	lockdef	 Lock 
	* \param	tileGraph	tiled graph	
	* \param	newLock	 ġ Lock
	* \param	sectionStartX	 Section   xǥ
	* \param	sectionStartY	 Section   yǥ
	* \return	 ڵ
	*/
	int				SubsequentProcessToLock(LockDef lockdef, TiledGraph &tileGraph, Lock newLock, int sectionStartX, int sectionStartY);

private:
	/** 
	* \brief class CTileArranger pointer 
	*/
	CTileArranger	*m_pTileArranger;

	/** 
	* \brief class CTileArranger Graph  κ  TiledGraph
	*/
	TiledGraph		m_tiledGraph;

	/** 
	* \brief  Lock 
	*/
	vector<Lock>	m_locks;

	/** 
	* \brief  Key 
	*/
	vector<Key>		m_keys;

	/** 
	* \brief ADFS() Լ Articulation Point Bridge   Ѵ.
	*
	*  Slot  켱 Ž  ° 湮Ǿ 
	*/
	int				m_dForDfs[MAX_SLOT_MATRIX_SIZE][MAX_SLOT_MATRIX_SIZE];

	/** 
	* \brief ADFS() Լ Articulation Point Bridge   Ѵ.
	*
	*  Slot  Ʈ Back Edge    Slot ּ m_dForDfs 
	*/
	int				m_lowForDfs[MAX_SLOT_MATRIX_SIZE][MAX_SLOT_MATRIX_SIZE];
	int				m_counterForDfs;

	/** 
	* \brief ADFS() Լ ߰ Artiulcation Point 
	*/
	vector<Coor>	m_artiPoints;

	/** 
	* \brief AllocateKey() ο ʺ 켱 Ž   Ѵ.
	*/
	bool			m_bVisitedBfs[MAX_SLOT_MATRIX_SIZE][MAX_SLOT_MATRIX_SIZE];
};


