#include "TileArranger.h"
#include <fstream>

void CTileArranger::ResetMatrix() 
{ 
	memset((void*) m_slotMatrix, 0, sizeof(m_slotMatrix)); 

	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			m_slotMatrix[i][j].tileindex = -1;
			m_slotMatrix[i][j].pathType = Path::Undefined;
		}
	}
}


int CTileArranger::Process(UserCondition userCondition, CTileContainer* pTileContainer) 
{
	int lastSeq = 0;
	int rtv = 0;
	m_userCondition = userCondition;
	m_pTileContainer = pTileContainer;
	ResetMatrix();

	lastSeq = GenMainPath(m_userCondition.turnOnMainPath, m_userCondition.lenOfMainPath);
	if (!lastSeq) 
		return LEVGEN_MAINPATH_BACKTRACK_FAILED;

	rtv = SetArtiPoints(lastSeq);
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	SetSection();

	rtv = BroadenMainPath(lastSeq);
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = ComplicateMap(lastSeq);
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = RandomlyConnectMargin();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	rtv = ArrangeTile();
	if (rtv != LEVGEN_NOERROR)
		return rtv;

	return lastSeq;
}

int CTileArranger::BroadenMainPath(int lastSequence)
{
	for (int i=0; i<=(int) m_artiPoints.size(); i++)		// section  iterate
	{
		int lowerArtiPoint = 0;	// Main path 󿡼  Section    sequence
		int upperArtiPoint = 0;	// Main path 󿡼  Section    sequence

		if (i == 0) 
		{
			lowerArtiPoint = 1;
		}
		else
		{
			lowerArtiPoint = m_artiPoints[i-1].sequence;
		}

		if (i == (int) (m_artiPoints.size()))
		{
			upperArtiPoint = lastSequence;
		}
		else
		{
			upperArtiPoint = m_artiPoints[i].sequence;
		}

		if (upperArtiPoint - lowerArtiPoint <= 1) continue;

		int curThickness = m_userCondition.sectionThickness[i];	//  section  ؾ ϴ β

		for (int thick = 0; thick < curThickness - 1; thick++)	//  ϴ βŭ ݺѴ.
		{
			int curSequence = lowerArtiPoint;
			Direction::Type lastExpandDir = Direction::End;

			//  sequence  شϴ slot  list  Ѵ.
			// (main path  Ǹ鼭 ϳ sequence شϴ slot  ʰ ȴ.)
			vector<Coor> currentSlots;
			vector<Coor> nextSlots;
			currentSlots.clear();
			nextSlots.clear();


			for (int j = 0; j < m_userCondition.height; j++)
			{
				for (int k = 0; k < m_userCondition.width; k++)
				{
					if (m_slotMatrix[j][k].sequence == curSequence
						&& (m_slotMatrix[j][k].pathType == Path::MainPath || m_slotMatrix[j][k].pathType == Path::ExtendedMP))
					{
						Coor startCoor;
						startCoor.x = k;
						startCoor.y = j;
						currentSlots.push_back(startCoor);
					}
				}
			}

			while(curSequence + 1 != upperArtiPoint)
			{
				//  sequence  ̵Ѵ.
				for (int j = 0; j < m_userCondition.height; j++)
				{
					for (int k = 0; k < m_userCondition.width; k++)
					{
						if (m_slotMatrix[j][k].sequence == curSequence + 1
							&& (m_slotMatrix[j][k].pathType == Path::MainPath || m_slotMatrix[j][k].pathType == Path::ExtendedMP))
						{
							Coor nextCoor;
							nextCoor.x = k;
							nextCoor.y = j;
							nextSlots.push_back(nextCoor);
						}
					}
				}

				if (nextSlots.empty())
				{
					return LEVGEN_INCORRECTLY_CONNECTED_MAINPATH;
				}

				curSequence++;
				currentSlots.clear();
				for (int j = 0; j < (int) (nextSlots.size()); j++)
				{
					currentSlots.push_back(nextSlots[j]);
				}
				nextSlots.clear();

				// Ȯ ȮѴ.
				if (rand() % 100 >= EXPAND_MAINPATH_PERCENT) continue;

				// currentSlots ϰ  ִ slot  list  .
				vector<CoorwDir> coorsExpandable;
				coorsExpandable.clear();

				// path  鼭  Ȯ   Ȯ  Ѵ.
				if (lastExpandDir != Direction::End)
				{
					for (int j = 0; j < (int) (currentSlots.size()); j++)
					{
						Direction::Type dir = lastExpandDir;
						int nextX = currentSlots[j].x + DELTAX[dir];
						int nextY = currentSlots[j].y + DELTAY[dir];
						if (IsValidCoor(nextX, nextY) 
							&& m_slotMatrix[nextY][nextX].sequence == 0)
						{
							CoorwDir expandableCoor;
							expandableCoor.x = nextX;
							expandableCoor.y = nextY;
							expandableCoor.dir = dir;
							coorsExpandable.push_back(expandableCoor);
						}
					}
				}

				if (coorsExpandable.empty())
				{
					for (int j = 0; j < (int) (currentSlots.size()); j++)
					{
						for (int k = 0; k < NDIRECTION; k++) 
						{
							Direction::Type dir = DIRECTIONS[k];
							int nextX = currentSlots[j].x + DELTAX[dir];
							int nextY = currentSlots[j].y + DELTAY[dir];
							if (IsValidCoor(nextX, nextY) 
								&& m_slotMatrix[nextY][nextX].sequence == 0)
							{
								CoorwDir expandableCoor;
								expandableCoor.x = nextX;
								expandableCoor.y = nextY;
								expandableCoor.dir = dir;

								coorsExpandable.push_back(expandableCoor);
							}
						}
					}
				}

				if (!coorsExpandable.empty())
				{
					// currentSlots ϰ ϳ Slot Ͽ ȮѴ.
					random_shuffle(coorsExpandable.begin(), coorsExpandable.end());
					
					m_slotMatrix[coorsExpandable[0].y][coorsExpandable[0].x].sequence = curSequence;
					m_slotMatrix[coorsExpandable[0].y][coorsExpandable[0].x].section = i + 1;
					m_slotMatrix[coorsExpandable[0].y][coorsExpandable[0].x].pathType = Path::ExtendedMP;
					lastExpandDir = coorsExpandable[0].dir;

					// Ȯ Slot ̿ Slot Ų.
					for (int k = 0; k < NDIRECTION; k++) 
					{
						Direction::Type dir = DIRECTIONS[k];
						int nextX = coorsExpandable[0].x + DELTAX[dir];
						int nextY = coorsExpandable[0].y + DELTAY[dir];
						if (IsValidCoor(nextX, nextY) 
							&& m_slotMatrix[nextY][nextX].sequence != 0 
							&& m_slotMatrix[nextY][nextX].section == i + 1)
						{
							m_slotMatrix[coorsExpandable[0].y][coorsExpandable[0].x].adjacencies[dir] = true;
							m_slotMatrix[nextY][nextX].adjacencies[3-dir] = true;
						}
					}
				}
			}
		}
	}

	return LEVGEN_NOERROR;
}

bool CTileArranger::FindCoorWithSequenceOnMainPath(int sequence, int *x, int *y)
{
	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			if (m_slotMatrix[i][j].pathType == Path::MainPath && m_slotMatrix[i][j].sequence == sequence)
			{
				*x = j;
				*y = i;
				return true;
			}
		}
	}

	return false;
}


void CTileArranger::SetSection()
{
	int cursor = 0;

	int curX = m_startX;
	int curY = m_startY;

	while(curX != m_endX || curY != m_endY)
	{
		for (int i = 0; i < NDIRECTION; i++) 
		{
			Direction::Type dir = DIRECTIONS[i];

			int nextX = curX+DELTAX[dir];
			int nextY = curY+DELTAY[dir];
			if (IsValidCoor(nextX, nextY) 
				&& m_slotMatrix[nextY][nextX].sequence == m_slotMatrix[curY][curX].sequence + 1)
			{
				curX = nextX;
				curY = nextY;
				break;
			}
		}

		if (m_slotMatrix[curY][curX].sequence == m_artiPoints[cursor].sequence) 
		{
			cursor++;
		}
		else
		{
			m_slotMatrix[curY][curX].section = cursor + 1;
		}
	}
}

int CTileArranger::SetArtiPoints(int lastSequence)
{
	m_artiPoints.clear();
	for (int i=0; i < m_userCondition.nOfLock; i++) 
	{
		ArtiPoint ap;
		ap.type = ArtiPoint::Lock;
		ap.sequence = 0;
		ap.index = i;

		m_artiPoints.push_back(ap);
	}
	for (int i=0; i < m_userCondition.nOfSokoban; i++) 
	{
		ArtiPoint ap;
		ap.type = ArtiPoint::Sokoban;
		ap.sequence = 0;
		ap.index = i;

		m_artiPoints.push_back(ap);
	}

	// lock sokoban  ϰ ´.
	random_shuffle(m_artiPoints.begin(), m_artiPoints.end());

	//   
	double interval = ((lastSequence-1-2*ARTIPOINT_MARGIN_AT_END) / (double) (m_artiPoints.size() + 1)); 

	for (vector<ArtiPoint>::iterator it=m_artiPoints.begin(); it != m_artiPoints.end(); it++)
	{
		int lowerBound = 0;
		int upperBound = 0;
		
		if (it==m_artiPoints.begin()) 
		{
			lowerBound = ARTIPOINT_MARGIN_AT_END + 1;
			if (lowerBound >= lastSequence)
				return LEVGEN_TOO_SHORT_MAINPATH_TO_ALLOC_ARTIPOINT;
		}
		else 
		{
			lowerBound=(*(it-1)).sequence + MINIMUM_SECTION_LENGTH;
			if (lowerBound >= lastSequence)
				return LEVGEN_TOO_SHORT_MAINPATH_TO_ALLOC_ARTIPOINT;
		}

		upperBound = (int) (interval * ( (int) (it - m_artiPoints.begin()) + 2)) + ARTIPOINT_MARGIN_AT_END;

		(*it).sequence = (rand() % (upperBound - lowerBound)) + lowerBound;
	}

	// sokoban   slot ǰ  ǥѴ.
	for (vector<ArtiPoint>::iterator it=m_artiPoints.begin(); it != m_artiPoints.end(); it++)
	{
		if ((*it).type == ArtiPoint::Sokoban)
		{
			int x,y;
			bool rbool = FindCoorWithSequenceOnMainPath((*it).sequence, &x, &y);
			if (!rbool)
				return LEVGEN_NO_SLOT_WITH_SEQUENCE;

			m_slotMatrix[y][x].isUsed = true;
			m_slotMatrix[y][x].isSokoban = true;
		}
	}

	return LEVGEN_NOERROR;
}


int CTileArranger::GenMainPath(int reqTurn, int reqLen) 
{
	m_startX = rand() % m_userCondition.width;
	m_startY = rand() % m_userCondition.height;
	
	ResetMatrix();

	return Dfs(m_startX, m_startY, Direction::End, 0, 1, reqTurn, reqLen);
}

bool CTileArranger::IsValidCoor(int curX, int curY) 
{
	if (curX >=0 && curX<m_userCondition.width && curY>=0 && curY<m_userCondition.height) 
		return true;
	else
		return false;
}

int CTileArranger::Dfs(int curX, int curY, Direction::Type curDir, 
							int curTurn, int curLen, 
							int reqTurn, int reqLen)
{
	//   ġ  ư.
	if (curTurn >= reqTurn + 1)
		return 0;

	m_slotMatrix[curY][curX].sequence = curLen;
	m_slotMatrix[curY][curX].pathType = Path::MainPath;

	//  ϴ Main Path  Ǿ ȯѴ.
	if (curTurn >= reqTurn && curLen >= reqLen) {
		m_endX = curX;
		m_endY = curY;
		return curLen;
	}

	deque<Direction::Type> validMoves;
	bool isStraightPossible = false;

	//  ̵  list .
	//      óѴ.
	for (int i = 0; i < NDIRECTION; i++) 
	{
		Direction::Type dir = DIRECTIONS[i];

		int nextX = curX+DELTAX[dir];
		int nextY = curY+DELTAY[dir];
		if (IsValidCoor(nextX, nextY) && (m_slotMatrix[nextY][nextX].sequence == 0)) {
			if (dir == curDir) isStraightPossible = true;
			else
				validMoves.push_back(dir);
		}
	}

	if (validMoves.empty() && !isStraightPossible)
	{
		m_slotMatrix[curY][curX].sequence = 0;
		m_slotMatrix[curY][curX].pathType = Path::Undefined;
		return 0;
	}

	if (isStraightPossible)
	{
		int randValue = rand() % 100;
		
		if (randValue < STRAIGHT_PRIORITY)		//   켱 Ѵ. 
		{
			random_shuffle(validMoves.begin(), validMoves.end());
			validMoves.push_front(curDir);
		}
		else
		{
			validMoves.push_back(curDir);
			random_shuffle(validMoves.begin(), validMoves.end());
		}
	}
	else
		random_shuffle(validMoves.begin(), validMoves.end());	//    ´.

	for (deque<Direction::Type>::iterator it = validMoves.begin(); it != validMoves.end(); it++)
	{
		Direction::Type dir = (*it);
		int nextX = curX+DELTAX[dir];
		int nextY = curY+DELTAY[dir];
		int nextTurn = curTurn;
		if (curDir != Direction::End && curDir != dir) nextTurn++;

		m_slotMatrix[curY][curX].adjacencies[dir] = true;
		m_slotMatrix[nextY][nextX].adjacencies[3-dir] = true;

		int lastSeq = Dfs(nextX, nextY, dir, nextTurn, curLen+1, reqTurn, reqLen);
		if (lastSeq > 0)
			return lastSeq;

		m_slotMatrix[curY][curX].adjacencies[dir] = false;
		m_slotMatrix[nextY][nextX].adjacencies[3-dir] = false;
	}

	m_slotMatrix[curY][curX].sequence = 0;
	m_slotMatrix[curY][curX].pathType = Path::Undefined;
	return 0;
}

void CTileArranger::PrintMatrix(ostream& os)
{
	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			bool flag = false;
			for (int k = 0; k < (int) m_artiPoints.size(); k++)
			{
				if (m_slotMatrix[i][j].sequence == m_artiPoints[k].sequence 
					&& (m_slotMatrix[i][j].pathType == Path::MainPath || m_slotMatrix[i][j].pathType == Path::ExtendedMP))
				{
					flag = true;
					break;
				}
			}
			if (!flag)
				os << m_slotMatrix[i][j].sequence;
			else
				os << "*";
			
			if (j != m_userCondition.width-1)
			{
				if (m_slotMatrix[i][j].adjacencies[Direction::East] == true)
					os << "-";
				else
					os << " ";
			}

			if (flag || m_slotMatrix[i][j].sequence < 10)
				os << "  ";
			else
				os << " ";
//			os << "\t";
		}
		os << endl;
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			if (m_slotMatrix[i][j].adjacencies[Direction::South])
				os << "|   ";
			else
				os << "    ";

//			os << " " << "\t";

		}
		os << endl;
	}

	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			if (m_slotMatrix[i][j].tileindex == -1) 
				os << "-1 ";
			else
				os << m_pTileContainer->m_tiles[m_slotMatrix[i][j].tileindex].id << " ";
		}
		os << endl;
	}
}

void CTileArranger::GetEdgeCoor(Direction::Type dir, int index, int *x, int *y)
{
	if (DELTAX[dir] == 0) *x = index;
	else if (DELTAX[dir] == -1) *x = 0;
	else if (DELTAX[dir] == 1) *x = (m_pTileContainer->m_tileWidth) - 1;
	if (DELTAY[dir] == 0) *y = index;
	else if (DELTAY[dir] == -1) *y = 0;
	else if (DELTAY[dir] == 1) *y = (m_pTileContainer->m_tileHeight) - 1;
}

float CTileArranger::EvalAdjRel(int x, int y, int tileIndex)
{
	float retValue = 0.0;
	for (int i = 0; i < NDIRECTION; i++) 
	{
		Direction::Type dir = DIRECTIONS[i];
		int adjX = x+DELTAX[dir];
		int adjY = y+DELTAY[dir];
		if (IsValidCoor(adjX, adjY)
			&& m_slotMatrix[adjY][adjX].tileindex != -1)
		{
			int size = 0;
			if (DELTAX[dir] != 0)
			{
				size = m_pTileContainer->m_tileWidth;
			}
			else
			{
				size = m_pTileContainer->m_tileHeight;
			}
			for (int j = 0; j < size; j++)
			{
				int xOfCurTile = 0; int yOfCurTile = 0;
				int xOfAdjTile = 0; int yOfAdjTile = 0;
				GetEdgeCoor(dir, j, &xOfCurTile, &yOfCurTile);
				GetEdgeCoor((Direction::Type) (3 - dir), j, &xOfAdjTile, &yOfAdjTile);
				int typeCurrent = m_pTileContainer->m_tiles[tileIndex].typeMatrix[yOfCurTile][xOfCurTile];
				int typeAdj = m_pTileContainer->m_tiles[m_slotMatrix[adjY][adjX].tileindex].typeMatrix[yOfAdjTile][xOfAdjTile];
				
				if (typeCurrent == typeAdj)
				{
					retValue += (float) 0.7;	//  cell  type   
				}
				else
				{
					if (typeCurrent > 0 && typeAdj > 0)			//  ġ cell 'ް'  0.3
					{
						retValue += (float) 0.3;
					}
					else if (typeCurrent > 0 && typeAdj < 0)
					{
						retValue += (float) 0.3;
					}
					else if (typeCurrent < 0 && typeAdj < 0)
					{
						retValue += (float) 0.1;
					}
					else if (typeCurrent < 0 && typeAdj > 0)	//  cell 'ް'  0.2
					{
						retValue += (float) 0.2;
					}
				}
			}
		}
	}	
	return retValue;
}

int CTileArranger::ComplicateMap(int lastSequence)
{
	for (int i=0; i<=(int) m_artiPoints.size(); i++)
	{
		int lowerArtiPoint = 0;
		int upperArtiPoint = 0;

		if (i == 0) 
		{
			lowerArtiPoint = 1;
		}
		else
		{
			lowerArtiPoint = m_artiPoints[i-1].sequence;
		}

		if (i == (int) (m_artiPoints.size()))
		{
			upperArtiPoint = lastSequence - 1;
		}
		else
		{
			upperArtiPoint = m_artiPoints[i].sequence;
		}

		if (upperArtiPoint - lowerArtiPoint <= 1) 
			return LEVGEN_INVALID_ALLOC_ARTIPOINT;

		int curDiff = m_userCondition.sectionDiffs[i];			//  Section  ̵
		int lenForDiff = MAX_LENGTH_FOR_SECTION_DIFF[curDiff];	// ̵  subpath 

		int minSubPathWOCycle = 0;								// ؾ ϴ "cycle ƴ subpath"  
		if (i != (int) (m_artiPoints.size()) && m_artiPoints[i].type == ArtiPoint::Lock)
		{
			// ø lock   ش ŭ cycle  subpath  ϴ .
			minSubPathWOCycle = m_userCondition.lockDefs[m_artiPoints[i].index].complexity;
		}

		for (int j = 0; j<NUM_SUBPATH_FOR_SECTION_DIFF[curDiff]; j++)
		{
			vector<Coor> candidates;	//  Slot Ȯ  Slot  .
			candidates.clear();

			for (int y = 0; y < m_userCondition.height; y++)
			{
				for (int x = 0; x < m_userCondition.width; x++)
				{
					if (m_slotMatrix[y][x].sequence != 0 && m_slotMatrix[y][x].section == i + 1)
					{
						for (int k = 0; k < NDIRECTION; k++) 
						{
							Direction::Type dir = DIRECTIONS[k];
							int nextX = x+DELTAX[dir];
							int nextY = y+DELTAY[dir];
							if (IsValidCoor(nextX, nextY) && m_slotMatrix[nextY][nextX].sequence == 0)
							{
								Coor newCoor;
								newCoor.x = x;
								newCoor.y = y;

								candidates.push_back(newCoor);
							}
						}

					}
				}
			}

			if (candidates.size() == 0)
			{
				break;	//  ƴϴ.  Ȯ   
			}

			// Ȯ  Slot߿ ϳ ϰ 
			int randsel = (int) (rand() % (candidates.size()));
			int orgSeq = 0;
			bool bCycleAllowed = true;


			Coor selCoor = candidates[randsel];

			// orgSeq  sub path ٿ Ǵ "main path slot" sequence
			//   ̿Ͽ 2*2 cycle Ǵ  ´.
			// (2*2 cycle Main Path о ȿ µ
			// ̹ Main Path ڰ  ̸ŭ  ܰ踦 Ʊ 
			//  ̻   ȴ.)
			//   
			// Main Path : 1 - 2 - (3) -<4>- 5 - 6
			// Sub Path :           |					 Sub Path orgSeq 3 Ǹ
			// Sub Path :           4  -[5]				[5]  Main Path <4>  ϴ   ʴ´.
			// Sub Path :           |
			// Sub Path :           5					
			if (m_slotMatrix[selCoor.y][selCoor.x].pathType == Path::MainPath
				|| m_slotMatrix[selCoor.y][selCoor.x].pathType == Path::ExtendedMP)
				orgSeq = m_slotMatrix[selCoor.y][selCoor.x].sequence;
			else if (m_slotMatrix[selCoor.y][selCoor.x].pathType == Path::SubPath)
				orgSeq = m_slotMatrix[selCoor.y][selCoor.x].orgSequence;
			else
				assert(false);

			if (j < minSubPathWOCycle)
			{
				bCycleAllowed = false;
			}

			MakeSubPath(selCoor.x, selCoor.y, Direction::End, lenForDiff, orgSeq, bCycleAllowed);
		}


	}

	return LEVGEN_NOERROR;
}

void CTileArranger::MakeSubPath(int x, int y, Direction::Type curDir, int maxLen, int orgSeq, bool bCycleAllowed)
{
	if (maxLen == 0)
		return;

	bool flag = false;
	Direction::Type nextDir = Direction::End;

	if (curDir != Direction::End && rand() % 100 < STRAIGHT_PRIORITY)
	{
		int nextX = x+DELTAX[curDir];
		int nextY = y+DELTAY[curDir];

		if (IsValidCoor(nextX, nextY) && m_slotMatrix[nextY][nextX].sequence == 0)
		{
			flag = true;
			nextDir = curDir;
		}
	}

	if (!flag)
	{
		//  move  list  .
		vector<Direction::Type> validMoves;
		validMoves.clear();
		for (int i = 0; i < NDIRECTION; i++) 
		{
			Direction::Type dir = DIRECTIONS[i];

			if (dir == curDir) continue;

			int nextX = x+DELTAX[dir];
			int nextY = y+DELTAY[dir];
			if (IsValidCoor(nextX, nextY) && (m_slotMatrix[nextY][nextX].sequence == 0)) {
				validMoves.push_back(dir);
			}
		}

		if (validMoves.empty()) return;

		flag = true;
		//  move ߿ random ϰ ϳ 
		nextDir = validMoves[rand() % validMoves.size()];

	}
	
	int nextX = x+DELTAX[nextDir];
	int nextY = y+DELTAY[nextDir];

	m_slotMatrix[nextY][nextX].sequence = m_slotMatrix[y][x].sequence + 1;
	m_slotMatrix[nextY][nextX].section = m_slotMatrix[y][x].section;
	m_slotMatrix[nextY][nextX].pathType = Path::SubPath;
	m_slotMatrix[nextY][nextX].orgSequence = orgSeq;

	m_slotMatrix[y][x].adjacencies[nextDir] = true;
	m_slotMatrix[nextY][nextX].adjacencies[(Direction::Type) (3 - nextDir)] = true;

	if (bCycleAllowed)
	{
		// ο slot   slot ̿ edge  .
		for (int i = 0; i < NDIRECTION; i++) 
		{
			Direction::Type dir = DIRECTIONS[i];

			int adjX = nextX+DELTAX[dir];
			int adjY = nextY+DELTAY[dir];
			if (IsValidCoor(adjX, adjY) 
				&& m_slotMatrix[adjY][adjX].section == m_slotMatrix[y][x].section
				&& m_slotMatrix[adjY][adjX].sequence != 0)
			{
				if (m_slotMatrix[adjY][adjX].pathType == Path::SubPath
					|| ( (m_slotMatrix[adjY][adjX].pathType == Path::MainPath || m_slotMatrix[adjY][adjX].pathType == Path::ExtendedMP)
						&& abs(m_slotMatrix[adjY][adjX].sequence - orgSeq) > 1)
						)
				{
					m_slotMatrix[nextY][nextX].adjacencies[dir] = true;
					m_slotMatrix[adjY][adjX].adjacencies[(Direction::Type) (3-dir)] = true;
				}
			}
		}
	}

	MakeSubPath(nextX, nextY, nextDir, maxLen - 1, orgSeq, bCycleAllowed);

}

int CTileArranger::RandomlyConnectMargin()
{
	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{
			Slot& slot = m_slotMatrix[i][j];
			if (slot.sequence != 0) continue;

			// slot ̿ slot   踦 Ѵ.
			// ̿ slot  path     .
			// ̿ slot  path   MARGIN_CONNECTION_PERCENT Ȯ 
			for (int k = 0; k < NDIRECTION; k++) 
			{
				Direction::Type dir = DIRECTIONS[k];
				int adjX = j+DELTAX[dir];
				int adjY = i+DELTAY[dir];
				if (IsValidCoor(adjX, adjY))
				{
					if (m_slotMatrix[adjY][adjX].sequence != 0)
					{
						slot.adjacencies[dir] = false;
						m_slotMatrix[adjY][adjX].adjacencies[(Direction::Type)(3-dir)] = false;
					}
					else// if (m_slotMatrix[adjY][adjX].tileindex == 0)
					{
						if ((rand() % 100) < MARGIN_CONNECTION_PERCENT) 
						{
							slot.adjacencies[dir] = true;
							m_slotMatrix[adjY][adjX].adjacencies[(Direction::Type)(3-dir)] = true;
						}
					}
				}
			}
		}
	}

	return LEVGEN_NOERROR;
}

int CTileArranger::ArrangeTile()
{
	for (int i = 0; i < m_userCondition.height; i++) 
	{
		for (int j = 0; j < m_userCondition.width; j++) 
		{

			Slot& slot = m_slotMatrix[i][j];
			// m_pTileContainer ο  Ÿ  迡  list  д.
			//    踦 2 ǥϸ 4bit ڰ ȴ.
			//  ڿ شϴ list Ѵ.
			int indexForCon = m_pTileContainer->GetIndexForCon(slot.adjacencies[Direction::North],
																slot.adjacencies[Direction::West],
																slot.adjacencies[Direction::East],
																slot.adjacencies[Direction::South]);

			vector<int>		*pAvailableTileList = NULL;
			if (slot.sequence != 0)
			{
				// m_listsForCon Ÿ ο  edgeκ ٸ edge    ִ ΰ ϴ Ÿ̴.
				//  
				// -1 -1  1 -1 -1
				// -1 -1  1 -1 -1			 Ÿ ʰ  Ա 
				//  1  1  1 -1 -1			 Ա  Ա ̿ Ÿ ο ΰ Ѵ.
				// -1 -1 -1 -1 -1
				// -1 -1 -1 -1 -1
				pAvailableTileList = &(m_pTileContainer->m_listsForCon[indexForCon]);		
			}
			else
			{
				// m_listsForConNoCross  m_listsForCon ߰ Ÿ  ΰ  ʴ Ÿϵ Ѵ.
				//  
				// -1 -1  1 -1 -1
				// -1 -1  1 -1 -1			 Ÿ Ͽ  Ա 
				//  1 -1  1 -1  1			 Ա ̿ ΰ .
				// -1 -1 -1 -1 -1
				// -1 -1  1 -1 -1
				pAvailableTileList = &(m_pTileContainer->m_listsForConNoCross[indexForCon]);
			}

			// sokobanable  tile  ִ ˻
			if (slot.isSokoban)
			{
				bool flag = false;
				for (int k = 0; k < (int) (pAvailableTileList->size()); k++)
				{
					int tileindex = (*pAvailableTileList)[k];
					if (m_pTileContainer->m_tiles[tileindex].isSokobanable)
					{
						flag = true;
						break;
					}
				}
				if (!flag)
				{
					return LEVGEN_NO_SUITABLE_SOKOBAN_TILE;
				}
			}

			vector<float>	relevanceVec;
			float			maxRelevance = -10000;
			int				aroundMaxCount = 0;
			for (int k = 0; k < (int) (pAvailableTileList->size()); k++)		// ġ  Ÿϵ " յ" غ.
			{
				float relevance = EvalAdjRel(j, i, (*pAvailableTileList)[k]);
				if (slot.isSokoban)
				{
					int tileindex = (*pAvailableTileList)[k];
					if (!(m_pTileContainer->m_tiles[tileindex].isSokobanable))
						relevance = -10000;
				}

				relevanceVec.push_back(relevance);
				if (relevance > maxRelevance)
				{
					maxRelevance = relevance;	
				}
			}

			if (relevanceVec.empty() || maxRelevance < 0) return LEVGEN_NO_SUITABLE_TILE;

			for (int k = 0; k < (int) (relevanceVec.size()); k++)
			{
				if (relevanceVec[k] >= maxRelevance * RELEVANCE_TOLERANCE_RATIO)
				{
					aroundMaxCount++;
				}
			}

			int	pick = rand() % aroundMaxCount;							// ִ " յ"   ̻ Ÿ ߿ ϰ 
			for (int k = 0; k < (int) (relevanceVec.size()); k++)
			{
				if (relevanceVec[k] >= maxRelevance * RELEVANCE_TOLERANCE_RATIO)
				{
					if (pick == 0)
					{
						slot.tileindex = (*pAvailableTileList)[k];
						break;
					}
					pick--;
				}
			}
		}
	}

	return LEVGEN_NOERROR;
}
