#include "UserCondition.h"
#include "..\common\tinyxml.h"

int ReadUserConditionXML(string fullpath, UserCondition *userCond)
{
	if (!userCond) 
		return LEVGEN_INVALIDARG;

	TiXmlDocument userCondXml(fullpath.c_str());
	if (!(userCondXml.LoadFile())) 
		return LEVGEN_FILEOPENERROR;

	TiXmlNode* root = userCondXml.FirstChild(LEVGEN_USERCOND_TAG_USERCONDITION);
	if (!root) 
		return LEVGEN_INVALIDUSERCOND;

	TiXmlElement* element = root->FirstChildElement(LEVGEN_USERCOND_TAG_TILESETXML);
	if (!element) 
		return LEVGEN_INVALIDUSERCOND;
	userCond->tilesetXMLPath = element->Attribute(LEVGEN_USERCOND_ATTR_PATH);

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_UNITXML);
	if (!element) 
		return LEVGEN_INVALIDUSERCOND;
	userCond->unitXMLPath = element->Attribute(LEVGEN_USERCOND_ATTR_PATH);

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_MAPSIZE);
	if (!element) 
		return LEVGEN_INVALIDUSERCOND;

	userCond->height = atoi(element->Attribute(LEVGEN_USERCOND_ATTR_HEIGHT));
	userCond->width = atoi(element->Attribute(LEVGEN_USERCOND_ATTR_WIDTH));

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_MAINPATH);
	if (!element)
		return LEVGEN_INVALIDUSERCOND;

	userCond->turnOnMainPath = atoi(element->Attribute(LEVGEN_USERCOND_ATTR_TURN));
	userCond->lenOfMainPath = atoi(element->Attribute(LEVGEN_USERCOND_ATTR_LENGTH));

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_LOCKDEF);
	if (!element)
		return LEVGEN_INVALIDUSERCOND;

	TiXmlElement* lockElement = element->FirstChildElement(LEVGEN_USERCOND_TAG_LOCK);
	if (!lockElement)
		return LEVGEN_INVALIDUSERCOND;

	while (lockElement)
	{
		LockDef newlockdef;
		newlockdef.numOfKey = atoi(lockElement->Attribute(LEVGEN_USERCOND_ATTR_NUMOFKEY));
		newlockdef.complexity = atoi(lockElement->Attribute(LEVGEN_USERCOND_ATTR_COMPLEXITY));
		userCond->lockDefs.push_back(newlockdef);

		lockElement = lockElement->NextSiblingElement(LEVGEN_USERCOND_TAG_LOCK);
	}
	userCond->nOfLock = (int) (userCond->lockDefs.size());

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_SOKOBANDEF);
	if (!element)
		return LEVGEN_INVALIDUSERCOND;


	TiXmlElement* sokobanElement = element->FirstChildElement(LEVGEN_USERCOND_TAG_SOKOBAN);
	if (!sokobanElement)
		return LEVGEN_INVALIDUSERCOND;

	while (sokobanElement)
	{
		SokobanDef newsokobandef;
		newsokobandef.difficulty = atoi(sokobanElement->Attribute(LEVGEN_USERCOND_ATTR_DIFFICULTY));
		userCond->sokobanDefs.push_back(newsokobandef);

		sokobanElement = sokobanElement->NextSiblingElement(LEVGEN_USERCOND_TAG_SOKOBAN);
	}
	userCond->nOfSokoban = (int) (userCond->sokobanDefs.size());

	//element = element->FirstChildElement(LEVGEN_USERCOND_TAG_SOKOBAN);
	//for (int i = 0; i < userCond->nOfSokoban; i++)
	//{
	//	if (!element)
	//		return LEVGEN_INVALIDUSERCOND;

	//	userCond->sokobanDiffs[i] = atoi(element->Attribute(LEVGEN_USERCOND_ATTR_DIFFICULTY));

	//	element = element->NextSiblingElement(LEVGEN_USERCOND_TAG_SOKOBAN);
	//}

	element = root->FirstChildElement(LEVGEN_USERCOND_TAG_SECTION);
	for (int i = 0; i < (userCond->nOfSokoban) + (userCond->nOfLock)+1; i++)
	{
		if (!element)
			return LEVGEN_INVALIDUSERCOND;

		TiXmlElement* subelement = element->FirstChildElement(LEVGEN_USERCOND_TAG_MAINPATH);
		if (!subelement) 
			return LEVGEN_INVALIDUSERCOND;
		userCond->sectionThickness[i] = atoi(subelement->Attribute(LEVGEN_USERCOND_ATTR_THICKNESS));

		subelement = element->FirstChildElement(LEVGEN_USERCOND_TAG_SUBPATH);
		if (!subelement) 
			return LEVGEN_INVALIDUSERCOND;
		userCond->sectionDiffs[i] = atoi(subelement->Attribute(LEVGEN_USERCOND_ATTR_DIFFICULTY));

		element = element->NextSiblingElement(LEVGEN_USERCOND_TAG_SECTION);
	}

	return LEVGEN_NOERROR;
}

