#include "..\common\common.h"
#include "UserCondition.h"
#include "TileArranger.h"
#include "..\common\TileContainer.h"
#include "KeyLockComposer.h"
#include "CelledMapGenerator.h"
#include <fstream>

void PrintError(int rtv)
{
	cout << rtv << endl;
}

int main()
{
	srand(static_cast<unsigned int>(time(0)));

	UserCondition userCondition;
	int rtv;

	rtv = ReadUserConditionXML("UserCondition.xml", &userCondition);
	if (rtv < 0)
	{
		PrintError(rtv);
		return 0;
	}

	CTileContainer tileContainer;
	tileContainer.Reset();
	rtv = tileContainer.ReadXML(userCondition.tilesetXMLPath);
	if (rtv < 0)
	{
		PrintError(rtv);
		return 0;
	}

	CTileArranger tileArranger;

	rtv = tileArranger.Process(userCondition, &tileContainer);
	if (rtv < 0)
	{
		PrintError(rtv);
		return 0;
	}

 	tileArranger.PrintMatrix(cout);

	CKeyLockComposer keyLockComposer;

	rtv = keyLockComposer.Process(&tileArranger);
	if (rtv < 0)
	{
		PrintError(rtv);
		return 0;
	}

	keyLockComposer.PrintMatrix(cout);

	CCelledMapGenerator celledMapGenerator;
	rtv = celledMapGenerator.Process(&keyLockComposer);
	if (rtv < 0)
	{
		PrintError(rtv);
		return 0;
	}

	ofstream fout;
	fout.open("Result.xml");
	celledMapGenerator.Export(fout);
	fout.close();

	return 0;
}
