// maindlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINDLG_H__BE0DD3EC_FC72_4B75_BF0F_F8BA63388ED1__INCLUDED_)
#define AFX_MAINDLG_H__BE0DD3EC_FC72_4B75_BF0F_F8BA63388ED1__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "..\common\common.h"
#include "UserCondition.h"
#include "TileArranger.h"
#include "..\common\TileContainer.h"
#include "KeyLockComposer.h"
#include "CelledMapGenerator.h"
#include <fstream>

class CMainDlg : public CDialogImpl<CMainDlg>//, public CTitleTipHandler<CMainDlg>, public CDragListNotifyImpl<CMainDlg>
{
public:
	enum { IDD = IDD_MAINDLG };

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		COMMAND_ID_HANDLER(ID_GEN, OnGen)
		COMMAND_ID_HANDLER(ID_BUTTON_SAVE, OnSave)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
		COMMAND_ID_HANDLER(IDC_BUTTON_OPENFILE, OnOpenFile)

		//	REFLECT_NOTIFICATIONS()
	END_MSG_MAP()

protected:
	CEdit			c_edInputFilename;
	CListViewCtrl	c_lvFrames;


	CButton			c_btGen;
	CButton			c_btSave;

private:
	UserCondition*			m_pUserCondition;
	CTileContainer*			m_pTileContainer;
	CTileArranger*			m_pTileArranger;
	CKeyLockComposer*		m_pKeyLockComposer;
	CCelledMapGenerator*	m_pCelledMapGenerator;

	int						m_msgCounter;

private:
	void ResetLevelObject(BOOL bIncludeInput = FALSE)
	{
		if (bIncludeInput && m_pUserCondition)
		{
			delete m_pUserCondition;
			m_pUserCondition = NULL;
		}
		if (bIncludeInput && m_pTileContainer)
		{
			delete m_pTileContainer;
			m_pTileContainer = NULL;
		}
		if (m_pTileArranger)
		{
			delete m_pTileArranger;
			m_pTileArranger = NULL;
		}
		if (m_pKeyLockComposer)
		{
			delete m_pKeyLockComposer;
			m_pKeyLockComposer = NULL;
		}
		if (m_pCelledMapGenerator)
		{
			delete m_pCelledMapGenerator;
			m_pCelledMapGenerator = NULL;
		}
	}
	void PrintError(int rtv)
	{
		char temp[1000];

		if (rtv == LEVGEN_FILEOPENERROR)
		{
			sprintf(temp, "Error\t: %s", "Cannot open file");
		}
		else if (rtv == LEVGEN_INVALIDTILESET)
		{
			sprintf(temp, "Error\t: %s", "Invalid tile set");
		}
		else if (rtv == LEVGEN_KEY_ALLOC_TO_CELL_FAILED)
		{
			sprintf(temp, "Message\t: %s", "Key Allocation to Cell failed");
		}
		else if (rtv == LEVGEN_MAINPATH_BACKTRACK_FAILED)
		{
			sprintf(temp, "Message\t: %s", "Mainpath Creation Backtracking failed");
		}
		else if (rtv == LEVGEN_NO_BRIDGE_TO_ALLOC_LOCK)
		{
			sprintf(temp, "Message\t: %s", "Lock Allocation to Bridge failed");
		}
		else if (rtv == LEVGEN_NO_SLOT_TO_ALLOC_KEY)
		{
			sprintf(temp, "Message\t: %s", "Key Allocatin to Slot failed");
		}
		else if (rtv == LEVGEN_SUBLOCK_ALLOC_FAILED)
		{
			sprintf(temp, "Message\t: %s", "Subordinate Lock Allocation failed");
		}
		else if (rtv == LEVGEN_KEY_ALLOC_FAILED)
		{
			sprintf(temp, "Message\t: %s", "Key Allocation failed");
		}
		else
		{
			sprintf(temp, "Error\t: errno = %d", rtv);
		}

		PrintMsg(temp);
	}

	void PrintMsg(char *msg)
	{
		c_lvFrames.AddItem(m_msgCounter, 0, msg);
		c_lvFrames.SelectItem(m_msgCounter);
		m_msgCounter++;
	}

	void ClearMsg()
	{
		c_lvFrames.DeleteAllItems();
		m_msgCounter = 0;
	}


public:

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		c_edInputFilename.Attach(GetDlgItem(IDC_INPUT_FILENAME));
		c_edInputFilename.SetWindowText(_T(""));

		c_lvFrames.Attach(GetDlgItem(IDC_MSGLIST));

		c_lvFrames.SetExtendedListViewStyle(LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT);
		c_lvFrames.InsertColumn(0, _T("Message"), LVCFMT_LEFT, 500, -1);

		c_btGen.Attach(GetDlgItem(ID_GEN));
		c_btSave.Attach(GetDlgItem(ID_BUTTON_SAVE));

		c_btGen.EnableWindow(FALSE);
		c_btSave.EnableWindow(FALSE);

		m_pUserCondition = NULL;
		m_pTileContainer = NULL;
		m_pTileArranger = NULL;
		m_pKeyLockComposer = NULL;
		m_pCelledMapGenerator = NULL;

		ClearMsg();

		return TRUE;
	}
	

	LRESULT OnGen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		//static int counter = 0;

		//char temp[1000];

		//sprintf(temp, "sample %d", counter);

		//
		//c_lvFrames.AddItem(counter, 0, temp);
		//c_lvFrames.SelectItem(counter);

		//counter++;

		int rtv = LEVGEN_NOERROR;
		bool flag = false;
		for (int i = 0; i < RETRY_COUNT; i++)
		{
			if (i != 0)
			{
				char msg[1000];

				sprintf(msg, "Message\t: Retrying %d...", i);
				PrintMsg(msg);
			}

			m_pTileArranger = new CTileArranger;
			m_pKeyLockComposer = new CKeyLockComposer;
			m_pCelledMapGenerator = new CCelledMapGenerator;

			rtv = m_pTileContainer->ReadXML(m_pUserCondition->tilesetXMLPath);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject();
				c_btSave.EnableWindow(FALSE);
				continue;
			}

			rtv = m_pTileArranger->Process(*m_pUserCondition, m_pTileContainer);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject();
				c_btSave.EnableWindow(FALSE);
				continue;
			}

 			//tileArranger.PrintMatrix(cout);

			rtv = m_pKeyLockComposer->Process(m_pTileArranger);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject();
				c_btSave.EnableWindow(FALSE);
				continue;
			}

			//keyLockComposer.PrintMatrix(cout);

			rtv = m_pCelledMapGenerator->Process(m_pKeyLockComposer);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject();
				c_btSave.EnableWindow(FALSE);
				continue;
			}

			c_btSave.EnableWindow(TRUE);

			PrintMsg("Message\t: Level Generation Succeed !!");
			flag = true;
			break;
		}

		if (!flag)
		{
			PrintMsg("Error\t: Level Generation Failed !!");
			c_btSave.EnableWindow(FALSE);
		}
		//ofstream fout;
		//fout.open("Result.xml");
		//celledMapGenerator.Export(fout);
		//fout.close();

		return 0;
	}

	LRESULT OnSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		char szBuffer[4096];
		::GetModuleFileName(NULL, szBuffer, 4096);

		string strModulePath = szBuffer;
		int nIndex = strModulePath.rfind('\\');

		string strDefaultPath = strModulePath.substr(0, nIndex);

		strDefaultPath+= "\\Result.xml";
 
		CFileDialog fileDlg (FALSE, _T("xml"), strDefaultPath.c_str(),
							OFN_HIDEREADONLY | OFN_FILEMUSTEXIST,
							_T("XML File\0*.xml\0") );
		 
		if ( IDOK == fileDlg.DoModal() )
		{
			ofstream fout;
			fout.open(fileDlg.m_szFileName);
			m_pCelledMapGenerator->Export(fout);
			fout.close();

			char msg[4096];
			sprintf(msg, "Message\t: Exported to %s", fileDlg.m_szFileName);
			PrintMsg(msg);
		}

		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		ResetLevelObject(TRUE);
		EndDialog(wID);
		return 0;
	}

	LRESULT OnOpenFile(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		char szBuffer[4096];
		::GetModuleFileName(NULL, szBuffer, 4096);

		string strModulePath = szBuffer;
		int nIndex = strModulePath.rfind('\\');

		string strDefaultPath = strModulePath.substr(0, nIndex);
 
		strDefaultPath+= "\\*.xml";


		CFileDialog fileDlg (true, "xml", strDefaultPath.c_str(),
							OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT,
							_T("XML File\0*.xml\0") );
		 
		if ( IDOK == fileDlg.DoModal() )
		{
			char msg[4096];
			ClearMsg();

			c_edInputFilename.SetWindowText(fileDlg.m_szFileName);
			c_btGen.EnableWindow(TRUE);
			ResetLevelObject(TRUE);

			sprintf(msg, "Input\t: Parsing User Condition in %s", fileDlg.m_szFileName);
			PrintMsg(msg);

			m_pUserCondition = new UserCondition;
			int rtv = ReadUserConditionXML(fileDlg.m_szFileName, m_pUserCondition);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject(TRUE);
				c_btGen.EnableWindow(FALSE);
				c_btSave.EnableWindow(FALSE);
				return 0;
			}

			sprintf(msg, "Input\t: Parsing Tile Info in %s", m_pUserCondition->tilesetXMLPath.c_str());
			PrintMsg(msg);

			m_pTileContainer = new CTileContainer;
			m_pTileContainer->Reset();
			rtv = m_pTileContainer->ReadXML(m_pUserCondition->tilesetXMLPath);
			if (rtv < 0)
			{
				PrintError(rtv);
				ResetLevelObject(TRUE);
				c_btGen.EnableWindow(FALSE);
				c_btSave.EnableWindow(FALSE);
				return 0;
			}

			sprintf(msg, "Input\t: Tile count = %d, width = %d, height = %d", 
							m_pTileContainer->GetTileCount(), 
							m_pTileContainer->GetTileWidth(), m_pTileContainer->GetTileHeight());
			PrintMsg(msg);
			sprintf(msg, "Input\t: Map width = %d, height = %d", 
							m_pUserCondition->width, m_pUserCondition->height);
			PrintMsg(msg);
			sprintf(msg, "Input\t: Mainpath length = %d, turn = %d", 
							m_pUserCondition->lenOfMainPath, m_pUserCondition->turnOnMainPath);
			PrintMsg(msg);
			sprintf(msg, "Input\t: # of locks = %d", m_pUserCondition->nOfLock);
			PrintMsg(msg);
			sprintf(msg, "Input\t: # of sokobans = %d", m_pUserCondition->nOfSokoban);
			PrintMsg(msg);
			sprintf(msg, "Input\t: # of sections = %d",
								m_pUserCondition->nOfLock + m_pUserCondition->nOfSokoban + 1);
			PrintMsg(msg);
			for (int i = 0; i < m_pUserCondition->nOfLock + m_pUserCondition->nOfSokoban + 1; i++)
			{
				sprintf(msg, "Input\t: sections %d, Difficulty = %d Thickness = %d ",
									i, m_pUserCondition->sectionDiffs[i], m_pUserCondition->sectionThickness[i]);
				PrintMsg(msg);
			}
			sprintf(msg, "Input\t: Parsing Completed");
			PrintMsg(msg);
		}

		return 0;
	}
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINDLG_H__BE0DD3EC_FC72_4B75_BF0F_F8BA63388ED1__INCLUDED_)
